#!/bin/ksh 

# -------------------------------------------------------------
# The following control variables are for development use only 
#
# The following verification control steps can be disabled
# without impacting critical function
# -------------------------------------------------------------
skipVerificationSteps='no' 

skipCheckFilesetUpdateLevels='no'
skipVerifyOperatingSystemLevel='no'
skipVerifySystemIsAnH70='no'
skipSanityCheckVolumeGroups='no'
skipListVolumeGroups='no'
skipAuditDisksFromReservedVgAsRequired='no'
skipVerifyTsmStuff='no'
skipCheck3466DiskResourcesAixViewOnly='no'
skipVerifySsaCabling='no'


# -------------------------------------------------------------
# The following steps control critical function 
# These steps can only be disabled with knowledge that the 
# critical function has already been completed
# -------------------------------------------------------------
skipReorderSsaPdisksAndHdisks='no'
skipReorderSsaPdisksAndHdisks='no'
skipExtendAdsmVg='no'
skipCreateJFS='no'
skipCreateADSMresource='no'
skipDefineADSM='no'
skipSaveHWinfo='no'

# -------------------------------------------------------------
# The following variables allow the use of this script on 
# 3466 Cx0 systems with 9GB drives. Not for customer use!
# -------------------------------------------------------------
export allow9GBdisks='no'
export allow9GBdisks='yes'

export allowH50='no'
export allowH50='yes'

#@
#@ *********************************************************************
#@
#@ Script Name: h27366a1.ec
#@
#@ Description: This is a mainline script program. This script controls
#@              the addition of SSA disk
#@
#@              Each mainline script sets must set up the proper
#@              environment:
#@                1) initialize environment variables from the
#@                   primary (required) and override (optiona)
#@                   environment control files
#@                2) Initialize master log name
#@                3) Invoke the script buildDiskSets.stsm to build the
#@                   diskSet<xx> environment variables.
#@
#@ Input:
#@
#@    Command Line Input:
#@
#@    StorageSmart Build Environment Variables: all
#@
#@ Output: variable dependent on the environment control variables
#@
#@ Change Cntl  Name                 Description
#@ -----------  ------------------   ----------------------------------
#@ 11/08/2000   Chris Knapp          Create
#@
#@ ********************************************************************

writeErr ()
{
  print $* >$2
} # end writeErr

enterOneToExit ()
{
  sleep 1 # seems to be required, or output gets messed up
  PS3=''
  select userResponse in \
      'Enter 1 followed by the ENTER key to exit'
  do
    case $REPLY in
      1) :
         echo
         echo 'User entered 1, exiting ...........'
         ;;
      *) echo
         echo "User entered: '$REPLY'. The only valid response is 1"
         echo
         echo 'Enter 1 followed by the ENTER key to continue'
         echo
         ;;
      esac

    if [[ -n $userResponse ]] ; then
      break
    fi
  done

  exit 1

} # end enterOneToExit

enterOneToContinue ()
{
  sleep 1 # seems to be required, or output gets messed up
  PS3=''
  select userResponse in \
      'Enter 1 followed by the ENTER key to continue'
  do
    case $REPLY in
      1) :
         echo
         echo 'User entered 1, continuing ...........'
         ;;
      *) echo
         echo "User entered: '$REPLY'. The only valid response is 1"
         echo
         echo 'Enter 1 followed by the ENTER key to continue'
         echo
         ;;
      esac

    if [[ -n $userResponse ]] ; then
      break
    fi
  done

} # end enterOneToContinue

enterOneToContinueTwoToExit ()
{
  sleep 1 # seems to be required, or output gets messed up
  PS3=''
  select userResponse in \
      'Enter 1 followed by the ENTER key to continue'\
      'Enter 2 followed by the ENTER key to exit'
  do
    case $REPLY in
      1) echo
         echo 'User entered 1, continuing ...........'
         ;;
      2) echo
         echo 'User entered 2, exiting ...........'
         echo 
         exit 1
         ;;
      *) echo
         echo "Invalid response: '$REPLY'."
         echo
         ;;
      esac

    if [[ -n $userResponse ]] ; then
      break
    fi
  done

} # end enterOneToContinueTwoToExit

promptForAmountOfNewDbSpace ()
{

  echo; echo "$stdSeperatorLine"
  thisSubroutine="promptForAmountOfNewDbSpace"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  export maxNumberOfDbVolumes=1

  # -------------------------------------------------------------
  # build list of hdisks that will be added
  # -------------------------------------------------------------
  volumeGroupForHdisksToBeAdded="$1" # this would have to be changed when different size disks are used
  shift 1
  theListOfHdisksToAdd="$*"

  # -------------------------------------------------------------
  # Display the list of hdisks to be added with this mes
  # -------------------------------------------------------------
  echo 
  echo "INFO ($scriptName) - the following hdisks, in $volumeGroupForHdisksToBeAdded, will be added with this MES"
  echo 
  displayCounter=0
  hdiskAddList=''
  for currentHdisk in $theListOfHdisksToAdd
  do
    hdiskAddList="$hdiskAddList $currentHdisk"
    displayCounter=$(($displayCounter + 1)) 
    if (( $(($displayCounter % 4)) == 0 ))
    then
      echo "  -- $hdiskAddList"
      displayCounter=0
      hdiskAddList=''
    fi
  done
  echo

  if (( $displayCounter > 0 ))
  then
    echo "  -- $hdiskAddList"
  fi

  # -------------------------------------------------------------
  # Verify the number of hdisks to add is a multiple of 4
  # -------------------------------------------------------------
  echo
  numberOfHdisksToAdd=`echo $theListOfHdisksToAdd | wc -w | awk '{print $1}'`
  verifyNumberOfDisksToAddIsMultipleOf4=$(($numberOfHdisksToAdd % $numberOfHdisksToAdd))
  echo
  if (( $verifyNumberOfDisksToAddIsMultipleOf4 != 0 )) || (( $numberOfHdisksToAdd < 4 ))
  then
    echo 
    echo "NSM-95405-E: ERROR: The number of hdisks to add '$numberOfHdisksToAdd' is invalid."
    echo "    The number of hdisks to add must be a multiple of 4. This is"
    echo "    a script error. Previous validation scripts should have prevented"
    echo "    this condition from occuring."
    echo 
    enterOneToExit
  fi

  # -------------------------------------------------------------
  # Use previous db selection if appropriatte
  # -------------------------------------------------------------
  if [ "$promptInformationMustBeObtainedFromProgressLog" = 'yes' ]
  then
    : # required information already initialized from the pre-existing Progress Log
  else
    # -------------------------------------------------------------
    # determine if any resources are already allocated on the
    # disks to be allocated with this MES
    # -------------------------------------------------------------
    echo 
    echo "INFO ($scriptName) - checking hdisks: .\c"
    lvAllocationList=''
    for currentHdisk in $theListOfHdisksToAdd
    do 
      lvAllocationList="$lvAllocationList `lspv -l $currentHdisk | grep -v $currentHdisk: | grep -v 'LV NAME' | awk '{print $1}'`"
      echo '.\c'
    done
  
    # -------------------------------------------------------------
    # build list of logical volumes that are on the disks to add
    # -------------------------------------------------------------
    lvsAlreadyAllocated=''
    for currentLvItem in $lvAllocationList
    do
      checkForNameMatchingJfslog=`echo $currentLvItem | grep loglv`
      if [ "$checkForNameMatchingJfslog" = '' ]
      then
        lvsAlreadyAllocated="$lvsAlreadyAllocated $currentLvItem"
        echo '.\c'
      fi
    done
    echo
    echo

    # -------------------------------------------------------------
    # Prompt user for the amount of DB space to add
    # -------------------------------------------------------------
    if [ "$lvsAlreadyAllocated" != '' ]
    then
      echo
      echo "NSM-95448-E: ERROR ($scriptName) the hdisks to be added"
      echo "  are not empty. It is possible that the customer has"
      echo "  has enabled disks prior to this MES. There is no"
      echo "  legitimate reason for this to have been done. Cleanup of"
      echo "  these resources may be required. Recommended action"
      echo "  is to review the resources with the customer to"
      echo "  better understand how they were allocated. Presumably,"
      echo "  the resources do not match 3466 naming requirements."
      echo 
      echo "  IBM support should be called to review the situation"
      echo "  if the customer is unable to provide assistance to"
      echo "  reconcile the existance of these resources."
      echo 

      
      # -------------------------------------------------------------
      # prompt user for action to take
      # -------------------------------------------------------------
      readyToContinue='no'
      while [ "$readyToContinue" = 'no' ]
      do
        echo
        echo '**********************************************************************'
        echo '*'
        echo "* Select option from the following list"
        echo '*'
        echo '**********************************************************************'
        sleep 1 # seems to be required, or output gets messed up
        PS3='
Enter Choice:'
        select userResponse in \
            'Display resources on the hdisks to be added' \
            'Exit'
        do
          case $REPLY in
            1) echo
               echo 'User entered 1, Display resources on the hdisks to be added'
  
              # -------------------------------------------------------------
              # determine if any DB resources are already allocated on 
              # these disks
              # -------------------------------------------------------------
              for currentHdisk in $theListOfHdisksToAdd
              do 
                numResourcesOnHdisk=`lspv -l $currentHdisk | grep -v $currentHdisk: | grep -v 'LV NAME' | wc -l | awk '{print $1}'`
                echo 
                if (( $numResourcesOnHdisk == 0 ))
                then
                  echo 
                  echo "INFO ($scriptName) - no resources are currently allocated on $currentHdisk"
                else
                  theCommand="lspv -l $currentHdisk"
                  echo "INFO ($scriptName) -  issuing command '$theCommand' to display resources on $currentHdisk"
                  echo
                  eval $theCommand
                fi
                echo 
                enterOneToContinue
              done
               ;;
            2) echo
               echo 'User entered 3, exiting .....'
               echo 
               enterOneToExit
               ;;
            *) echo
               echo 'Invalid Selection.'
               ;;
            esac
    
          if [[ -n $userResponse ]] ; then
            break
          fi
        done # done select
      done # while
    fi
  fi
  
  # ----------------------------------------------------------
  # no previously allocated resources on hdisks being added
  # ----------------------------------------------------------
  echo 
  echo "INFO ($scriptName) - collecting information about hdisks: .\c"

  # ------------------------------------------------------------
  # verify the PPs available on each disk are the same
  # ------------------------------------------------------------
  checkIfAdsmvgExists=`lsvg adsmvg2 2>/dev/null`
  ppSizeInMBsForCurrentVolumeGroup=`lsvg $volumeGroupForHdisksToBeAdded  | grep 'PP SIZE' | awk '{print $6}'`
  listOfAvailablePPs=''
  for currentHdisk in $theListOfHdisksToAdd
  do
    freePPsOnDisk=`lspv $currentHdisk | grep 'FREE PPs' | awk '{print $3}'`
    totalPPsOnDisk=`lspv $currentHdisk | grep 'TOTAL PPs' | awk '{print $3}'`
    listOfAvailablePPs="$listOfAvailablePPs $currentHdisk:$freePPsOnDisk:$totalPPsOnDisk"
  echo ".\c"
  done
  echo

  # -----------------------------------------------------------------
  # calculate information required for the db space prompt
  # -----------------------------------------------------------------
  firstHdiskToAdd=`echo $listOfAvailablePPs | awk '{print $1}' | cut -f1 '-d:'`
  totalPpsOnFirstHdiskToAdd=`echo $listOfAvailablePPs | awk '{print $1}' | cut -f3 '-d:'`
  freePpsOnFirstHdiskToAdd=`echo $listOfAvailablePPs | awk '{print $1}' | cut -f2 '-d:'`
  countOfHdisksToAdd=0
  for currentEntryCombo in $listOfAvailablePPs
  do
    currentHdisk=`echo $currentEntryCombo | cut -f1 '-d:'`
    countOfHdisksToAdd=$(($countOfHdisksToAdd + 1))
    freePpOnCurrentHdisk=`echo $currentEntryCombo | cut -f2 '-d:'`
    totalPpOnCurrentHdisk=`echo $currentEntryCombo | cut -f3 '-d:'`
    if (( $totalPpOnCurrentHdisk == $totalPpsOnFirstHdiskToAdd ))
    then
      :
    else
      echo
      echo "NSM-95444-E: ERROR ($scriptName) - the number of PPs on the current hdisk, $currentHdisk,"
      echo "    to be added. Do not match the number of PPs on the first hdisk, $firstHdiskToAdd, to"
      echo "    be added. Since previous validation verified the hdisks are empty, each hdisk"
      echo "    should have the same number of PPs available."      
      echo
      echo "  One possible cause for this is disks that are not the same size. This may"
      echo "    not have been identified in previous validation steps." 
      echo 
      echo "  Displaying information for the current and first hdisks to be added: "

      echo 
      theCommand="lspv  $firstHdiskToAdd"
      echo "Issuing command: '$theCommand'"
      echo "----------------------------------------------------------------------"
      eval $theCommand

      echo 
      theCommand="lspv  $currentHdisk"
      echo "Issuing command: '$theCommand'"
      echo "----------------------------------------------------------------------"
      eval $theCommand

      echo 
      echo "Exiting ...."
      echo
      enterOneToExit
    fi
  done

  if [ "$checkIfAdsmvgExists" != '' ]
  then
    numberOfDisksInAdsmvg2=`lsvg -p adsmvg2 | grep hdisk | wc -l | awk '{print $1}'`
  else
   numberOfDisksInAdsmvg2=0
  fi
  sizeOfHdiskInMb=$(($totalPpsOnFirstHdiskToAdd * $ppSizeInMBsForCurrentVolumeGroup))
  gbsOnAnHdisk=$(($sizeOfHdiskInMb / 1024)) 
  oneLessThanGbsOnAnHdisk=$(($gbsOnAnHdisk - 1))


  if [ "$promptInformationMustBeObtainedFromProgressLog" = 'yes' ]
  then
    : # other4packDBalloc already set
  else
    echo 
    echo "********************************************************************************"
    echo "*"
    echo "* The amount of TSM DB space to be added must be specified. This information "
    echo "* should be provided by the customer prior to installing this MES. If"
    echo "* the customer has not or chooses not to supply this information you"
    echo "* should specify to add no new DB space. "
    echo "*"
    echo "* The specification for TSM DB space is for primary DB space."
    echo "*"
    echo "* The amount of space to add is for PRIMARY TSM DB volume. The specification"
    echo "* will control the size of the filesystem which contains the DB volumes."
    echo "* The actual amount of space available for the DB volume is smaller than"
    echo "* the space allocated for the filesystem. This is due to overhead for JFS"
    echo "* and for TSM."
    echo "*"
    echo "* The space for PRIMARY TSM DB volume can be specified as:"
    echo "*"
    echo "*  -- One DB volume using all of the available space on the disk"
    echo "*  -- An amount in gigabytes to add."
    echo "*"
    echo "********************************************************************************"
    echo 
    enterOneToContinue
    echo 
    echo "Specify the amount of Primary DB space to add:"
    echo 
    validSpecification='no'
    while [ "$validSpecification" = 'no' ]
    do
      sleep 1 # seems to be required, or output gets messed up
      PS3='
Enter Choice:
'
      select userResponse in \
          "Add no new space for TSM DB volumes
"\
          "Use 1 full disk for primary DB space
"\
          "Specify the amount of primary space to add in gigabytes
   in the range of 1 to $oneLessThanGbsOnAnHdisk gigabytes.
"\
          'Exit'
      do
        case $REPLY in
          1) echo
             echo 'User entered 1, "Add no new space for TSM DB volumes"'
             unitForSpaceToAdd='none'
             ;;
          2) echo
             echo 'User entered 2, "Use 1 full disk for primary DB space"'
             unitForSpaceToAdd='disk'
               ;;
          3) echo
             echo 'User entered 3, "Specify the amount of space to add in gigabytes"'
             unitForSpaceToAdd='gigabyte'
               ;;
          4) echo
             echo 'User entered 4, exiting .....'
             echo 
             enterOneToExit
             ;;
          *) echo
             echo 'Invalid Selection.'
             ;;
        esac
        if [[ -n $userResponse ]] ; then
          break
        fi
      done # done select

      if [ "$unitForSpaceToAdd" = 'none' ]
      then
        other4packDBalloc=0
        validSpecification='yes'

      elif [ "$unitForSpaceToAdd" = 'disk' ]
      then
        other4packDBalloc=$sizeOfHdiskInMb
        unitsToAllocate=1
        validSpecification='yes'

      elif [ "$unitForSpaceToAdd" = 'gigabyte' ]
      then
        echo  
        echo "ENTER the number of gigabytes to add in the range from 1 to $oneLessThanGbsOnAnHdisk"
        read unitsToAllocate
        if [ "$unitsToAllocate" = '' ]
        then
          validSpecification='no'
        elif [ "$unitsToAllocate" -ge '1' ] && [ "$unitsToAllocate" -le "$oneLessThanGbsOnAnHdisk" ]
        then
          other4packDBalloc=$(($unitsToAllocate * 1024))
          validSpecification='yes'
        else
          validSpecification='no'
        fi
   
        if [ "$validSpecification" = 'no' ]
        then 
          echo 
          echo "NSM-95446-E: ERROR ($scriptName) - Invalid specification for the number"
          echo "    of gigabytes of PRIMARY DB space to add. User specified: '$unitsToAllocate'."
          echo 
          echo "  Try again, specifying a valid number"
          echo
          enterOneToContinue
        fi
      fi

      if [ "$validSpecification" = 'yes' ]
      then 
        if [ "$unitForSpaceToAdd" = 'none' ]
        then
          echo 
          echo "INFO ($scriptName) - confirm that no additional DB space will be allocated"
          echo 

        elif [ "$unitForSpaceToAdd" = 'disk' ]
        then
          echo 
          echo "INFO ($scriptName) - confirm the following DB allocation specification is correct:"
          echo 
          echo "   Use one full disk for additional PRIMARY TSM DB space"
          echo 
        else
          echo 
          echo "INFO ($scriptName) - confirm the following DB allocation specification is correct:"
          echo 
          echo "   Additional PRIMARY TSM DB space: $unitsToAllocate $unitForSpaceToAdd(s)"
          echo 
        fi

        sleep 1 # seems to be required, or output gets messed up
        PS3='
Enter Choice: 
'
        select dbConfirmation in \
            'DB allocation specifications are correct'\
            'DB allocation specifications are NOT correct, let me try again'
        do
          case $REPLY in
            1) echo
               echo 'User entered 1, "DB allocation specifications are correct"'
               ;;
            2) echo
               echo 'User entered 2, "DB allocation specifications are NOT correct, let me try again"'
               validSpecification='no'
               echo 
               enterOneToContinue
               echo 
               ;;
            *) echo
               echo "Invalid response, user entered: '$REPLY'."
               echo
               ;;
          esac
 
          if [[ -n $dbConfirmation ]] ; then
            break
          fi
        done
      fi
  
    done # while
  fi

  thisSubroutine="promptForAmountOfNewDbSpace"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"

} # promptForAmountOfNewDbSpace


sanityCheckVolumeGroups ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="sanityCheckVolumeGroups"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # --------------------------------------------------------
  # set flags
  # --------------------------------------------------------
  reservedVgExists='no'
  adsmvgExists='no'
  adsmvg2Exists='no'
  unknownVgExists='no'

  # --------------------------------------------------------
  # capture vg and disk information
  # --------------------------------------------------------
  volumeGroupList=`lsvg`
  numberOfSsaHdisks=`lsdev -Cc disk | grep SSA | wc -l | awk '{print $1}'`
  
  # --------------------------------------------------------
  # Recognize volume groups that exist
  # --------------------------------------------------------
  offlineVgList=''
  numberOfHdisksInReservedVg=0
  numberOfHdisksInAdsmvg=0
  numberOfHdisksInAdsmvg2=0
  for currentVg in $volumeGroupList
  do
    case $currentVg in

      ('reservedVg') reservedVgExists='yes'
         checkIfVgExistsAndActive=$(lsvg -o  | grep $currentVg )
         if [ "$checkIfVgExistsAndActive" != '' ]
         then
           numberOfHdisksInReservedVg=`lsvg -p $currentVg | grep hdisk | wc -l | awk '{print $1}'`
         else
           offlineVgList="$offlineVgList $currentVg"
         fi
         : ;;

      ('adsmvg') adsmvgExists='yes'
         checkIfVgExistsAndActive=$(lsvg -o  | grep $currentVg )
         if [ "$checkIfVgExistsAndActive" != '' ]
         then
           numberOfHdisksInAdsmvg=`lsvg -p $currentVg | grep hdisk | wc -l | awk '{print $1}'`
         else
           offlineVgList="$offlineVgList $currentVg"
         fi
         : ;;

      ('adsmvg2') adsmvg2Exists='yes'
         checkIfVgExistsAndActive=$(lsvg -o  | grep $currentVg )
         if [ "$checkIfVgExistsAndActive" != '' ]
         then
           numberOfHdisksInAdsmvg2=`lsvg -p $currentVg | grep hdisk | wc -l | awk '{print $1}'`
         else
           offlineVgList="$offlineVgList $currentVg"
         fi
         : ;;

      ('rootvg') : ;; # ignore

      (*) echo
          echo "NSM-95438-E: ERROR ($scriptName) - unknown volume group '$currentVg' exists"
          echo   
          echo " The only valid volume groups are. adsmvg, adsmvg2 and reservedVg"
          echo  
          echo " The unknown volume group must be removed before continuing"
          echo 
          enterOneToExit
          : ;;
    esac
  done

  if [ "$offlineVgList" != '' ]
  then
    echo
    echo "NSM-95424-E: ERROR - ($scriptName) -  one or more volume groups are varied"
    echo "  offline. Corrective action required. This script cannot proceed until all"
    echo "  volume group are varied online: The following volume groups exist"
    echo "  but are varied offline:"
    echo
    for currentOfflineVg in $offlineVgList
    do 
      echo "  -- $currentOfflineVg"
    done
    echo 
    echo "Exiting ..."
    echo 
    enterOneToExit

  else
    # --------------------------------------------------------
    # validate volume group and disk counts
    # --------------------------------------------------------
    vgValidationError='no'

    # -----------------------------------------------
    # audit a 48 disk system 
    # -----------------------------------------------
    if (( $numberOfSsaHdisks == 48 ))
    then
      if [ "$adsmvgExists" = 'yes' ]
      then
       : 
      else
        vgValidationError='yes'
      fi

      # ----------------------------------------------
      # report vg validation error
      # ----------------------------------------------
      if [ "$vgValidationError" = 'yes' ]
      then
        echo 
        echo "NSM-95440-E: ERROR ($scriptName) - SSA validation error for system with 48 disks"
        echo 
        echo "   The adsmvg volume group does not exist.  Corrective action required."
        echo
      fi
 
    # -----------------------------------------------
    # audit a 32 disk system 
    # -----------------------------------------------
    elif (( $numberOfSsaHdisks == 32 ))
    then
      if [ "$adsmvgExists" = 'yes' ]
      then
        :
      else
        vgValidationError='yes'
      fi
  
      # ----------------------------------------------
      # report vg validation error
      # ----------------------------------------------
      if [ "$vgValidationError" = 'yes' ]
      then
        echo 
        echo "NSM-95441-E: ERROR ($scriptName) - SSA validation error for system with 32 disks"
        echo 
        echo "   The adsmvg volume group does not exist.  Corrective action required."
        echo
      fi

    # -----------------------------------------------
    # audit a 16 disk system 
    # -----------------------------------------------
    elif (( $numberOfSsaHdisks == 16 ))
    then
      if [ "$adsmvg2Exists" = 'no' ] && [ "$adsmvgExists" = 'yes' ]
      then
        if  [ "$reservedVgExists" = 'yes' ]
        then
           if (( $numberOfHdisksInAdsmvg == 4 )) && (( $numberOfHdisksInReservedVg == 12 ))
           then
             : # this is valid
           elif (( $numberOfHdisksInAdsmvg == 8 )) && (( $numberOfHdisksInReservedVg == 8 ))
           then
             : # this is valid
          else
            vgValidationError='yes'
          fi
        else
          if (( $numberOfHdisksInAdsmvg == 16 ))
          then
            : # this is correct
          else
            vgValidationError='yes'
          fi
        fi
      else
        vgValidationError='yes'
      fi

      # ----------------------------------------------
      # report vg validation error
      # ----------------------------------------------
      if [ "$vgValidationError" = 'yes' ]
      then
        echo 
        echo "NSM-95442-E: ERROR ($scriptName) - SSA validation error for system with 16 disks"
        echo 
        echo "  The following configuration is supported for a 16 disk system:"
        echo 
        echo "    adsmvg must contain exactly 4 disks and reservedVg must contain exactly 12 disks -OR-"
        echo "    adsmvg must contain exactly 8 disks and reservedVg must contain exactly 8 disks -OR-"
        echo "    adsmvg must contain exactly 16 disks with no reservedVg volume group"
        echo
        echo "  The adsmvg volume group contains '$numberOfHdisksInAdsmvg'"
        echo "  The reservedVg volume group contains '$numberOfHdisksInReservedVg'"
        echo 
        echo "  Corrective action required."
        echo
      fi

    else
      echo 
      echo "NSM-95439-E: ERROR ($scriptName) - the number of SSA hdisks recognized by the "
      echo "  system is:  '$numberOfSsaHdisks'. This number is invalid."
      echo
      echo "  The system must have exactly 16, 32 or 48 SSA hdisks."
      echo 
      echo "  Corrective action required."
      echo 
      enterOneToExit
    fi

    # ----------------------------------------------
    # report vg validation error
    # ----------------------------------------------
    if [ "$vgValidationError" = 'yes' ]
    then
        echo 
        enterOneToExit
    fi
  fi

  thisSubroutine="sanityCheckVolumeGroups"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"

} # end sanityCheckVolumeGroups

listVolumeGroups ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine=""
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # --------------------------------------------------------
  # capture vg and disk information
  # --------------------------------------------------------
  volumeGroupList=`lsvg | grep -v rootvg`
  for currentVolumeGroup in $volumeGroupList
  do
    echo 
    echo "INFO ($scriptName) - Listing information for volume group: $currentVolumeGroup"

    echo 
    theCommand="lsvg $currentVolumeGroup"
    echo "Issuing command: '$theCommand'"
    echo
    eval $theCommand

    echo 
    theCommand="lsvg -l $currentVolumeGroup"
    echo "Issuing command: '$theCommand'"
    echo
    eval $theCommand

    echo 
    theCommand="lsvg -p $currentVolumeGroup"
    echo "Issuing command: '$theCommand'"
    echo
    eval $theCommand

    hdiskListForVg=`lsvg -p $currentVolumeGroup | grep hdisk | awk '{print $1}'`
    for currentHdisk in $hdiskListForVg
    do
      echo 
      theCommand="lspv $currentHdisk"
      echo "Issuing command: '$theCommand'"
      echo
      eval $theCommand
    done
    for currentHdisk in $hdiskListForVg
    do
      echo 
      theCommand="lspv -l $currentHdisk"
      echo "Issuing command: '$theCommand'"
      echo
      eval $theCommand
    done

  done
  
  thisSubroutine="listVolumeGroups"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"

} # end listVolumeGroups

checkResourcesOnHdiskInReservedVg ()
{
  theHdiskToCheck="$1"
  theHiskId=`echo $theHdiskToCheck | cut -b6-`

  # -------------------------------------------------------
  # build resource list for all of the resources on the hdisk
  # first identify all of the logical volumes
  # -------------------------------------------------------
  lvlist=`lspv -l $theHdiskToCheck | grep -v DISTRIBUTION | grep -v $theHdiskToCheck':' | awk '{print $1}'`

  # -------------------------------------------------------
  # There must be EXACTLY one logical volume on this hdisk
  # It must follow the naming convention hdisk3reserved
  # whre the numeric portion is the numeric ID of the hdisk 
  # -------------------------------------------------------
  validationError='no'
  numberOfResourcesOnHdisk=`echo $lvlist | wc -w | awk '{print $1}'`

  if (( $numberOfResourcesOnHdisk == 0 ))
  then
    echo
    echo "NSM-95445-W: WARNING ($scriptName) - there are no resources allocated"
    echo "   on $theHdiskToCheck in reservedVg. This is an indicator that:"
    echo 
    echo "  1) this resource was used for some unathorized purpose"
    echo 
    echo "  2) a previous invocation of this script removed this resource"
    echo "     but did not complete the process of adding it to the"
    echo "     adsmvg volume group"
    echo
    enterOneToContinue

    if [ "$validationError" = 'no' ]
    then
      validationError='maybe'
    fi
  
  elif (( $numberOfResourcesOnHdisk > 1 ))
  then
    validationError='yes'
  fi

  # -------------------------------------------------------
  # formulate valid resource name for this disk
  # -------------------------------------------------------
  validLvName='hdisk'"$theHiskId"'reserved'

  # -------------------------------------------------------
  # validate each resource
  # -------------------------------------------------------
  for currentResource in $lvlist
  do
    filesystemIndicator=`lspv -l $theHdiskToCheck | grep -v DISTRIBUTION |  grep -v $theHdiskToCheck':' | grep $currentResource' ' | awk '{print $5}'`
    if [ "$currentResource" = "$validLvName" ]
    then
      if [ "$filesystemIndicator" = "N/A" ]
      then
        : # this is the only valid condition
      else
        validationError='yes'
      fi
    else
      validationError='yes'
    fi

  done

  # ------------------------------------------------------
  # Report if any validation errors occured
  # ------------------------------------------------------
  if [ "$validationError" = 'no' ]
  then
    returnCode=0
  elif [ "$validationError" = 'maybe' ] 
  then
    returnCode=1
  else
    returnCode=2
    echo
    echo "NSM-95406-E: ERROR - ($scriptName) - a resource was found on $theHdiskToCheck that is not valid."
    echo "       The only valid resource is single logical volume with the name: '$validLvName'." 
    echo "       No filesystem can be associatted with this logical volume"
    echo
    theCommand="lspv -l $theHdiskToCheck"
    echo "       Issuing command: '$theCommand' to list all resources on $theHdiskToCheck"
    echo 
    eval $theCommand
    echo
    enterOneToContinue
  fi
  
  return $returnCode         

} # end checkResourcesOnHdiskInReservedVg

verifyCommunicationWithTsmServer ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="verifyCommunicationWithTsmServer"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  tsmServerCommunicatedWith=1
  while (( $tsmServerCommunicatedWith != 0 ))
  do
    . vadmin $vadminQuitOption $adminIDin $adminPasswordIn
    tsmServerCommunicatedWith=$?
    echo
    if (( $tsmServerCommunicatedWith == 0 ))
    then
      echo "INFO ($scriptName) - communication with the TSM server verified"
    else
      echo "NSM-95443-W: WARNING ($scriptName) - unable to communicate with the TSM server"
      echo 
      echo "  Possible causes:"
      echo 
      echo '  -- The server is down. Instructions state it should be up prior'
      echo '     to running this script'
      echo 
      echo '  -- The administrator userid or password is incorrect'
      echo 
      echo '  -- The shared memory stanza, ss-m, is missing or invalid'
      echo '     Check the file "/usr/tivoli/tsm/client/ba/bin/dsm.sys"'
      echo  
      echo '  -- The shared memory communication protocol is not enabled'
      echo '     Try issuing the "dsmadmc" command manually from another window'
      echo '     Verify that the shared memory communication protocol is enabled'
      echo '     in the server options file, "/usr/tivoli/tsm/server/bin/dsmserv.opt"'
      echo
      theCommand='ps -eo "%p %c" | grep dsmserv' 
      echo "  Entering the command '$theCommand' to check if the server is up."
      echo
      eval $theCommand  | grep -v grep 
      echo 
      echo "  If the server is up results similar to following should have been"
      echo "  displayed immediately preceding this message."
      echo
      echo "  Example output from ps command: '23724 dsmserv' "
      echo 
      echo "  Corrective action required:"
      echo
      echo "  If the server is not up, open a seperate window and start the server"
      echo "  with the 'startadsm' command. If the server is up, other causes"
      echo "  for the communication failure should be investigated"
      echo
      echo "  Enter '1' followed by the enter key to continue. The communication"
      echo "      test with the TSM server will be tried again."
      echo 
      echo "  Enter CNTL-C if you choose to terminate this script"
      echo
      enterOneToContinue
      echo
    fi
  done

  thisSubroutine="verifyCommunicationWithTsmServer"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end verifyCommunicationWithTsmServer
 
installUpdates ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="installUpdates"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # -------------------------------------------------------------
  # cd to the image directory
  # -------------------------------------------------------------
  theCommand="cd /cd/usr/sys/inst.images"
  echo
  echo "Issuing command: $theCommand"
  eval $theCommand
 
  # -------------------------------------------------------------
  # Install the filesets in the installation directory
  # Note: the filesets may exist at the same or later level in which case
  # the install will NO-OP
  # -------------------------------------------------------------
  echo
  theCommand=" /usr/lib/instl/sm_inst installp_cmd -a -Q -d '.' -f '_all_latest'  '-c' '-N' '-g' '-X'   '-G'"
  echo
  echo "Issuing command: $theCommand"
  eval $theCommand
 
  # -------------------------------------------------------------
  # return to the invocation directory
  # -------------------------------------------------------------
  cd - >/dev/null

  thisSubroutine="installUpdates"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end installUpdates

checkFilesetUpdateLevels ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="checkFilesetUpdateLevels"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # -------------------------------------------------------------
  # cd to the image directory
  # -------------------------------------------------------------
  theCommand="cd /cd/usr/sys/inst.images"
  echo
  echo "Issuing command: $theCommand"
  eval $theCommand
 
  # -------------------------------------------------------------
  # verify the required filesets are installed at the proper level
  # -------------------------------------------------------------
  $mesBuildscrDir/checkFilesetLevelsInDir
  theRC=$?
  if [ $theRC != 0 ]
  then
    echo
    echo "NSM-95408-E: ERROR - ($scriptName) - one or more filesets in the"
    echo "  installation director '`pwd`' are not installed at"
    echo "  the required level."
    echo 
    echo "Exiting ..."
    echo
    enterOneToExit
  fi
 
  # -------------------------------------------------------------
  # return to the invocation directory
  # -------------------------------------------------------------
  cd - >/dev/null

  thisSubroutine="checkFilesetUpdateLevels"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # checkFilesetUpdateLevels

# ---------------------------------------
# Get list of primary disk storage pools
# ---------------------------------------
getPrimaryDiskStoragePoolList ()
{
  theCommand="$xa -commadelimit select STGPOOL_NAME,ACCESS from STGPOOLS where DEVCLASS=\'DISK\' and POOLTYPE=\'PRIMARY\'"
  theResults=`eval $theCommand | grep -i -E "READ|WRITE|UNAVA"`
  diskpoolList=''
  for currentCombo in $theResults
  do
    diskpoolList="$diskpoolList `echo $currentCombo | cut -f1 -d','`"
  done

  echo "$diskpoolList" # return the results via STDOUT

} # end getPrimaryDiskStoragePoolList

verifyTsmStuff ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="verifyTsmStuff"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # ---------------------------------------
  # Set verification level flags
  # ---------------------------------------
  allVerifiedOK=0
  warningVerificationSet=1
  errorVerificationSet=2

  verificationLevel="$allVerifiedOK"

  # ---------------------------------------
  # Verify all db volumes are synched
  # ---------------------------------------
  echo; echo "$stdSeperatorLine"
  theCommand="$xa -commadelimit select COPY1_NAME,COPY1_STATUS,COPY2_NAME,COPY2_STATUS from DBVOLUMES"
  echo
  echo "INFO ($scriptName) - Issuing command: $theCommand"
  echo
  theResults=`eval $theCommand | grep '/db'`
  echo 
  echo "INFO ($scriptName) - TSM DB Mirror status follows:"
  for currentCombo in $theResults
  do
    copy1_name=`echo $currentCombo | cut -f1 -d','`
    copy1_status=`echo $currentCombo | cut -f2 -d','`
    copy2_name=`echo $currentCombo | cut -f3 -d','`
    copy2_status=`echo $currentCombo | cut -f4 -d','`
    if [ "$copy1_status" = "Sync'd" ] && [ "$copy2_status" = "Sync'd" ]
    then
      echo
      echo "  TSM DB Volume set, $copy1_name and $copy2_name are properly Sync'd"
    else
      verificationLevel="$warningVerificationSet"
      echo
      echo "NSM-95401-W: WARNING ($scriptName) - problem with mirrored TSM DB set:"
      echo  
      echo "    copy 1: dbvolume name: '$copy1_name' dbvolume status: '$copy1_status'"
      echo "    copy 2: dbvolume name: '$copy2_name' dbvolume status: '$copy2_status'"
      echo 
      echo "  The script phase being executed is: $userSelectedPhase"
      echo 
      echo "  TSM DB volumes that are not synched properly during phase 1, indicate"
      echo "  a pre-existing condition that should be corrected prior to continuing."
      echo  
      echo "  TSM DB volumes that are not synched properly during phase 2, may result"
      echo "  from a previous execution of this script not completing. If this is the"
      echo "  cause, they will be synched once this script completes. As with phase 1,"
      echo "  this may represent a pre-existing condition."
      echo
      echo "  Pre-existing conditions should be corrected BEFORE continuting!"
      echo
      enterOneToContinueTwoToExit
    fi
  done

  # ---------------------------------------
  # Verify all log volumes are synched
  # ---------------------------------------
  echo; echo "$stdSeperatorLine"
  theCommand="$xa -commadelimit select COPY1_NAME,COPY1_STATUS,COPY2_NAME,COPY2_STATUS from LOGVOLUMES"
  echo
  echo "INFO ($scriptName) - Issuing command: $theCommand"
  echo
  theResults=`eval $theCommand | grep '/lg'`
  echo 
  echo "INFO ($scriptName) - TSM Log Mirror status follows:"
  for currentCombo in $theResults
  do
    copy1_name=`echo $currentCombo | cut -f1 -d','`
    copy1_status=`echo $currentCombo | cut -f2 -d','`
    copy2_name=`echo $currentCombo | cut -f3 -d','`
    copy2_status=`echo $currentCombo | cut -f4 -d','`
    if [ "$copy1_status" = "Sync'd" ] && [ "$copy2_status" = "Sync'd" ]
    then
      echo
      echo "  TSM Log Volume set, $copy1_name and $copy2_name are properly Sync'd"
    else
      verificationLevel="$warningVerificationSet"
      echo
      echo "NSM-95402-W: WARNING ($scriptName) - problem with mirrored TSM Log set:"
      echo  
      echo "    copy 1: logvolume name: '$copy1_name' logvolume status: '$copy1_status'"
      echo "    copy 2: logvolume name: '$copy2_name' logvolume status: '$copy2_status'"
      echo 
      echo "  TSM Log volumes that are not synched properly result from a pre-existing"
      echo "  condition. This script does not allocate new TSM log volumes."
      echo
      echo "  This condition should be corrected BEFORE continuting!"
      echo
      enterOneToContinueTwoToExit
    fi
  done

  # ---------------------------------------
  # Verify at least one disk storage pool exists
  # ---------------------------------------
  echo; echo "$stdSeperatorLine"
  primaryDiskStoragePoolList=`getPrimaryDiskStoragePoolList`
  numberOfPrimaryDiskStoragePools=`echo $primaryDiskStoragePoolList | wc -w | awk '{print $1}'`
  echo
  if (( $numberOfPrimaryDiskStoragePools == 0 ))
  then
    verificationLevel="$errorVerificationSet"
    echo "NSM-95403-E: ERROR ($scriptName) - no primary disk storage pools are found"
    echo "        at least one primary storage pool must exist"
    echo  
  else
    echo "INFO ($scriptName) - The following primary disk storage pools exist:"
    echo 
    for currentStoragePool in $primaryDiskStoragePoolList
    do
      echo "  $currentStoragePool"
    done
    echo
  fi

  if [ "$verificationLevel" = "$errorVerificationSet" ]
  then
    echo
    echo "One or more ERRORs occurred. Corrective action required"
    echo 
    enterOneToExit
  elif [ "$verificationLevel" = "$warningVerificationSet" ]
  then
    echo
    echo 
    echo "**********************************************************************"
    echo '*'
    echo "* One or more WARNINGs were issued. Corrective action may be required"
    echo "* Review errors with customer or IBM support before continuing"
    echo '*'
    echo "* ENTER 1 followed by the ENTER key to continue -OR- CNTL-C to exit"
    echo '*'
    echo "**********************************************************************"
    enterOneToContinue
    echo
  else
    : # all verification passed
  fi

  thisSubroutine="verifyTsmStuff"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # verifyTsmStuff 

selectDiskPoolToAddTo ()
{
  echo; echo "$stdSeperatorLine"

  # ---------------------------------------
  # build list of primary disk storage pools
  # ---------------------------------------
  echo; echo "$stdSeperatorLine"
  primaryDiskStoragePoolList=`getPrimaryDiskStoragePoolList`
  numberOfPrimaryDiskStoragePools=`echo $primaryDiskStoragePoolList | wc -w | awk '{print $1}'`
  echo
  if (( $numberOfPrimaryDiskStoragePools == 1 ))
  then
    diskPoolToUse="$primaryDiskStoragePoolList"
  else
    validSpecification='no'
    numberOfEntriesInSelectionList=`echo $primaryDiskStoragePoolList | wc -w | awk '{print $1}'`
    primaryDiskStoragePoolList="$primaryDiskStoragePoolList abort/exit"
    while [ "$validSpecification" = 'no' ]
    do
      echo 
      echo "**********************************************************************"
      echo "*"
      echo "* INFO ($scriptName) - more than one primary disk storage pools"
      echo "*      exist on this system. Select one primary disk storage pool"
      echo "*      to use. All of the new disk storage pool volumes will be added"
      echo "*      to the disk storage pool that is selected. Disk storage pool"
      echo "*      volumes can be moved later."
      echo '*'
      echo '*  The following example moves disk pool volume /bfaa1dk2 to '
      echo '*  diskpool2'
      echo "*"
      echo '*  -- query vol, decide what disk volumes to move to the new domain'
      echo '*  -- update vol /bfaa1dk2/bf access=readonly, prevent any new data'
      echo '*          from being written to the diskpool volume.'
      echo '*  -- move data /bfaa1dk2/bf, moves data to other disk volumes within'
      echo '*          the storage pool in which it currently resides'
      echo '*  -- query vol /bfaa1dk2/bf, make sure it is not utilized'
      echo '*  -- delete vol /bfaa1dk2/bf, the volume will not be deleted unless'
      echo '*          it is empty'
      echo '*  -- define vol diskpool2 /bfaa1dk2/bf, define the volume to the'
      echo '*          new storage pool'
      echo "*"
      echo "**********************************************************************"
      echo 
      sleep 1
      PS3="
Enter choice: "
      select diskPoolToUse in \
         $primaryDiskStoragePoolList
      do
        if [ "$diskPoolToUse" = '' ]
        then
          echo "User selected option $REPLY, invalid selection"
 
        elif  [ "$REPLY" -le "$numberOfEntriesInSelectionList" ] && [ "$REPLY" -gt "0" ]
        then
          echo "User selected option $REPLY, $diskPoolToUse"
          validSpecification='yes'
        else
          echo
          echo "User selected option $REPLY, abort and exit"
          programRc=1
          echo 
          enterOneToExit
        fi
  
        if [[ -n $diskPoolToUse ]] ; then
          break
        fi
      done
  
      echo 
      echo "INFO ($scriptName) - confirm that all of the new disk storage pool"
      echo "     volumes, should be added to $diskPoolToUse."
      echo
      sleep 1 # seems to be required, or output gets messed up
      PS3='
Enter Choice: 
'
      select stgpoolSelection in \
          "Disk storage pool, $diskPoolToUse, is the correct storage pool"\
          "Disk storage pool, $diskPoolToUse, is NOT the correct storage pool"
      do
        case $REPLY in
          1) echo
             echo "User entered 1, 'Storage pool, $diskPoolToUse, is the correct storage pool'"
             ;;
          2) echo
             echo "User entered 1, 'Storage pool, $diskPoolToUse, is NOT the correct storage pool'"
             validSpecification='no'
             echo 
             enterOneToContinue
             echo 
             ;;
          *) echo
             echo "Invalid response, user entered: '$REPLY'."
             echo
             ;;
        esac

        if [[ -n $stgpoolSelection ]] ; then
          break
        fi
      done # select

    done # while [ "$validSpecification" = 'no' ]
  fi

} # selectDiskPoolToAddTo

initializeStuff ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="initializeStuff"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # -------------------------------------------------------------
  # Create a unique build ID
  # -------------------------------------------------------------
  if [ "$1" = '' ]
  then
    buildid=$(date "+%m%d%Y_%H%M%S")  # yymmdd_hhmmss
  else
    buildid="$1"
  fi
 
  logid=$(hostname)
  firstdrawer=""
  lastdrawer=""
  serverId="ss-m"
  buildControlFile="/cd/template.master"
 
  # -------------------------------------------------------------
  # Make sure master template exists
  # -------------------------------------------------------------
 
  echo
  if [[ ! -f $buildControlFile ]]
  then
    echo "NSM-95409-E: ERROR - ($scriptName) - The master build control file: '$buildControlFile'"
    echo "       does not exist. Call IBM Support. "
    echo 
    echo "Exiting ...."
    echo
    enterOneToExit
  else
    echo "INFO: ($scriptName) - Initializing control variables from master control file"
    echo
    echo "Issuing command:  '. $buildControlFile'"
    . $buildControlFile  # run the master template
  fi
 
  # -------------------------------------------------------------
  # Execute template override as appropriatte
  # -------------------------------------------------------------
  overrideTemplateFile='' # disable, most values now hard codeed in script
  echo
  if [ "$overrideTemplateFile" = '' ]
  then
    :
#   echo
#   echo "INFO: ($scriptName) - no override template specified"
  elif [ -f "$overrideTemplateFile" ]
  then
    echo
    echo "INFO: ($scriptName) -Override template specified"
    echo
    echo "Issuing command:  '. $overrideTemplateFile'"
    . $overrideTemplateFile
  else
    echo
    echo "NSM-95410-E: ERROR - ($scriptName) -override template: $overrideTemplateFile not found"
    echo
    echo "Exiting ...."
    echo
    enterOneToExit
  fi
 
  # -------------------------------------------------------------
  #  Over-rides
  # -------------------------------------------------------------
  export buildType='ec'                # force ec build to be on
  export buildID="$buildid"
  export uniqueLogDir=$logid           # over-ride templates !!!
 
  # -------------------------------------------------------------
  # export DSM_LOG to direct error message to the log directory
  # -------------------------------------------------------------
  export DSM_LOG=$logDir

  thisSubroutine="initializeStuff"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end initializeStuff

verifyOperatingSystemLevel ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="verifyOperatingSystemLevel"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  theOslevelString=`oslevel`
 
  minRequiredLevel1=4
  minRequiredLevel2=3
  minRequiredLevel3=3
 
  oslevel1=`echo $theOslevelString | cut -f1 -d'.'`
  oslevel2=`echo $theOslevelString | cut -f2 -d'.'`
  oslevel3=`echo $theOslevelString | cut -f3 -d'.'`
 
  oslevelCheckPassed='no'
  if [ $oslevel1 -gt $minRequiredLevel1 ]
  then
    oslevelCheckPassed='yes' # os at greater than required level
  elif [ $oslevel1 -lt $minRequiredLevel1 ]
  then
    : # fall through, oslevel less than required
  else
    if [ $oslevel2 -gt $minRequiredLevel2 ]  # os at greater than required level
    then
      oslevelCheckPassed='yes'
    elif [ $oslevel2 -lt $minRequiredLevel2 ]
    then
      : # fall through, oslevel less than required
    else
      if [ $oslevel3 -gt $minRequiredLevel3 ]  # os at greater than required level
      then
        oslevelCheckPassed='yes'
      elif [ $oslevel3 -lt $minRequiredLevel3 ]
      then
        : # fall through, oslevel less than required
      else
        oslevelCheckPassed='yes' # os at min required level
      fi
    fi
  fi
 
  echo
  echo "INFO: ($scriptName) - the operating system level is: $theOslevelString"
  if [ "$oslevelCheckPassed" = 'no' ]
  then
    echo "INFO: ($scriptName) - the operating system level is not at least the required level"
    echo "         the minimum required level which is $minRequiredLevel1.$minRequiredLevel2.$minRequiredLevel3"
    echo
    echo "Exiting ...."
    echo
    enterOneToExit
  else
    echo "      this is >= to the minimum required level is $minRequiredLevel1.$minRequiredLevel2.$minRequiredLevel3"
  fi

  thisSubroutine="verifyOperatingSystemLevel"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end verifyOperatingSystemLevel

setPdiskType ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="setPdiskType"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # determine what type of disk we are talking about
  echo
  echo "INFO: ($scriptName) - Verifying that the pdisk type is 18GB"

  pdiskList=`lsdev -Cc pdisk | grep Available | awk '{print $1}'`
  firstAvailablePdisk=`echo $pdiskList | awk '{print $1}'`
  if [ "$firstAvailablePdisk" = '' ]
  then
    echo
    echo "NSM-95411-E: ERROR - ($scriptName) - available pdisk not found. Corrective action required"
    echo
    echo "Exiting ...."
    echo
    enterOneToExit
  else
    str2=$(lscfg -vl $firstAvailablePdisk | grep "Machine Type and Model" | grep 09B)
    str3=$(lscfg -vl $firstAvailablePdisk | grep "Machine Type and Model" | grep 18B)
    if [ "$str2" != '' ]
    then
      if [ "$allow9GBdisks" = 'yes' ]
      then
        export diskType="9GB"
        export hdiskType="jbod_9"
      else
        echo
        echo "NSM-95412-E: ERROR - ($scriptName) -  This has one or more pdisks that are not 18GB"
        echo
        echo "Exiting ....."
        echo
        enterOneToExit
      fi
    elif [ "$str3" != '' ]
    then
      export diskType="18GB"
      export hdiskType="jbod_18"
    else
      echo
      echo "NSM-95413-E: ERROR - ($scriptName) -  This has one or more pdisks that are not 18GB"
      echo
      echo "Exiting ....."
      echo
      enterOneToExit
    fi
  fi

  thisSubroutine="setPdiskType"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end setPdiskType

verifySystemIsAnH70 ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="verifySystemIsAnH70"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  echo
  echo "INFO: ($scriptName) - verifying the system is an H70"

  systemIsAsupportedType='yes' # assume a match
  str=$(uname -m | cut -c 9,10 )
  if [ "$str" = "4C" ]
  then
    str=$(lsattr -El sys0 | grep modelname | grep H50)
    if [ "$str" = '' ]
    then
      str=$(lsattr -El sys0 | grep modelname | grep H70)
      if [ "$str" = '' ]
      then
        systemIsAsupportedType='no' # system is an H10 or some other unrecognized system
      else
        system="C01"
        systemType="release3"
        export processorType="H70"
      fi
    else
      if [ "$allowH50" = 'yes' ]
      then
        system="CXX Version 3"
        systemType="release3"
        export processorType="H50"
      else
        systemIsAsupportedType='no' # system is an H50
      fi
    fi
 else
   systemIsAsupportedType='no' # system is an R20 or some other unrecognized system
 fi
 
 if [ "$systemIsAsupportedType" != 'yes' ]
 then
    echo
    echo "NSM-95414-E: ERROR - ($scriptName) -  The RS/6000 model is not"
    echo "    recognized. This script can only be run on a 3466 C01 which"
    echo "    uses an RS/6000 H70."
    echo 
    echo "Exiting ..."
    echo
    enterOneToExit
 fi

  thisSubroutine="verifySystemIsAnH70"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end verifySystemIsAnH70

reorderSsaPdisksAndHdisks()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="reorderSsaPdisksAndHdisks"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"
  echo 

  # -------------------------------------------------
  # Invoke fixSSA to check if pdisks Hdisks are ordered correctly
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  echo
  fixSSA -q 
  fixSsaRc=$?
  echo
  if (( $fixSsaRc == 0 ))
  then
    echo "INFO ($scriptName) - hdisks / pdisks are named correctly" 
  else
 
     echo
     echo "INFO: ($scriptName) - hdisks / pdisks are NOTnamed correctly"
     echo "      invoke fixSSA to rename disks"
 
     # first halt the server
     echo
     echo "INFO: ($scriptName) - Halting sever so we can export the adsmvg volume group(s)"
     $xa halt
 
     # now export the volume group -- can't change disks if vg active
     echo
     echo "INFO: ($scriptName) - Exporting the adsmvg volume group(s). This may take a while."
     cmd="prebackup "
     echo
     echo "Issuing command: $cmd"
     eval $cmd
 
     # now reorder the disks
     cmd="fixSSA $logDir/fixSSA.out"
     echo
     echo "Issuing command: $cmd"
     eval $cmd
     rc=$?
     if [ "$rc" != 0 ]
     then
       echo "NSM-95415-E: ERROR - ($scriptName) -  Could not reorder disks. fixSSA is failing."
       echo "             Resolve problem or contact support."
       echo
       echo "Exiting ..."
       echo
       enterOneToExit
     fi
 
     # now pull everything back in
     echo
     echo "INFO: ($scriptName) - Importing the adsmvg volume group(s). This may take a while."
     cmd="postbackup "
     echo
     echo "Issuing command: $cmd"
     eval $cmd
 
     echo
     echo "If this MES crosses 2 drawer boundary, postbackup will fail"
     echo "because it thinks there should be an adsmvg2 but there isn't yet ..."
     echo "so deal with this problem and vary on the adsmvg and remount just"
     echo "in case this was a problem!!"
     theCommand="varyonvg adsmvg"
     echo "Issuing command: $theCommand"
     eval $theCommand
 
     echo
     theCommand="mount -a 1> /dev/null 2>&1"
     echo "Issuing command: $theCommand"
     eval $theCommand
 
 
     # Now restarting server
     echo
     echo "INFO: ($scriptName) - restarting ADSM."
     theCommand='startadsm all'
     echo "INFO: ($scriptName) - Issuing command: $theCommand"
     eval $theCommand
     echo "INFO: ($scriptName) - Waiting for ADSM server to restart."
     checkADSM.stsm
     rc=$?
     if [ "$rc" != 0 ]
     then
       echo "NSM-95416-E: ERROR - ($scriptName) -  Can not contact ADSM server."
       echo "     Fix problem and restart."
       echo
       echo "Exiting ..."
       echo
       enterOneToExit
     fi
  fi
 
 
  # -------------------------------------------------------------
  # Do some error checking!  Don't continue if we don't have
  # the volume group present ... that could destroy data!!!
  # -------------------------------------------------------------
  str=$(lsvg adsmvg 2> /dev/null)
  if [ "$str" = '' ]
  then
    echo "NSM-95417-E: ERROR - ($scriptName) -  adsmvg is not present."
    echo "     Do not continue until problem is resolved"
    echo
    echo "Exiting ..."
    echo
    enterOneToExit
  fi
 
  if [ "$diskInSystemBeforeMes" -gt 32 ]
  then
     str=$(lsvg adsmvg2 2> /dev/null)
     if [ "$str" = '' ]
     then
       echo "NSM-95418-E: ERROR - ($scriptName) -  adsmvg2 is not present."
       echo "     Do not continue until problem is resolved"
       echo
       echo "Exiting ..."
       echo
       enterOneToExit
     fi
  fi

  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end reorderSsaPdisksAndHdisks


displayDiskAdditionPrompt ()
{

  displayLine ()
  {
  cat << !*
$lineToPrint
!*
  } # end displayLine

  displayHeaderLines ()
  {
    if (( $loopCounter == 1 ))
    then
      clear 
      echo
      echo '********************************************************************************'
      echo '*'
      echo "* Information will be displayed on the following screens that will"
      echo "*   not fit in a window which was launched with the default size."
      echo '*'
      echo "* If you have not already increased the size of the window, do do now."
      echo '*'
      echo "* The window size can be increased by:"
      echo '*'
      echo "*  -- Maximizing to full screen with the maximize button"
      echo "*     (upper right hand corner)"
      echo '*'
      echo "*  -- Stretching the window height. Position the mouse on the"
      echo "*     top or bottom edge of the screen. Notice that the pointer"
      echo "*     changes to indicate the mouse is positioned to stretch the screen."
      echo "*     Hold down the left mouse key and move the mouse to stretch the"
      echo "*     screen."
      echo '*'
      echo "* Hit the Enter Key to continue when the screen size is adjusted"
      echo '*'
      echo '********************************************************************************'
      read
    fi

    # -------------------------------------------------------------------
    # Display header lines
    # -------------------------------------------------------------------
    clear
    lineToPrint="$sepSolid";   displayLine
    lineToPrint="$mainHeader"; displayLine
    lineToPrint="$sepSolid";   displayLine
    lineToPrint="$opHeader1";  displayLine
    lineToPrint="$opHeader2";  displayLine
    lineToPrint="$opHeader3";  displayLine
  } # end displayHeaderLines

  setOptionValues ()
  {
    case $optionSelected in 

      1) diskInSystemBeforeMes='4' 
         diskAddedWithThisMes='4'
         firstJbod4packForEc='2'
         lastJbod4packForEc="2"
         : ;;

      2) diskInSystemBeforeMes='4' 
         diskAddedWithThisMes='12'
         firstJbod4packForEc='2'
         lastJbod4packForEc="4"
         : ;;

      3) diskInSystemBeforeMes='4' 
         diskAddedWithThisMes='28'
         firstJbod4packForEc='2'
         lastJbod4packForEc="8"
         : ;;

      4) diskInSystemBeforeMes='4' 
         diskAddedWithThisMes='44'
         firstJbod4packForEc='2'
         lastJbod4packForEc="12"
         : ;;

      5) diskInSystemBeforeMes='8' 
         diskAddedWithThisMes='8'
         firstJbod4packForEc='3'
         lastJbod4packForEc="4"
         : ;;

      6) diskInSystemBeforeMes='8' 
         diskAddedWithThisMes='24'
         firstJbod4packForEc='3'
         lastJbod4packForEc="8"
         : ;;

      7) diskInSystemBeforeMes='8' 
         diskAddedWithThisMes='40'
         firstJbod4packForEc='3'
         lastJbod4packForEc="12"
         : ;;

      8) diskInSystemBeforeMes='16' 
         diskAddedWithThisMes='16'
         firstJbod4packForEc='5'
         lastJbod4packForEc="8"
         : ;;

      9) diskInSystemBeforeMes='16' 
         diskAddedWithThisMes='32'
         firstJbod4packForEc='5'
         lastJbod4packForEc="12"
         : ;;

     10) diskInSystemBeforeMes='32' 
         diskAddedWithThisMes='16'
         firstJbod4packForEc='9'
         lastJbod4packForEc="12"
         : ;;

     (*) echo
         echo "NSM-95419-E: ERROR ($scriptName)  - invalid option specified."
         echo "  This is a script error,  previous validation steps should have"
         echo "  prevented this sub-routine from being executed"
         echo 
         echo "Exiting ...."
         echo 
         enterOneToExit
         : ;;

    esac

  } # end setOptionValues

  promptForOptionSelection ()
  {
    echo
    echo "Enter the option from the row in the table that describes"
    echo '  the number of disks in the system prior to the MES and '
    echo '  the number of disks being added with this MES'
    echo
    echo "Enter the numeric option from the table, followed by ENTER: \c"
    read optionSelected
    echo
    echo "The user selected '$optionSelected'"
    echo 

  } # end promptForOptionSelection

  verifyUserSelection ()
  {
    
    verifySelectedOptionIsValid=`echo " $validOptionList " | grep " $optionSelected "`
    if [ "$verifySelectedOptionIsValid" = '' ]  || [ "$optionSelected" = '' ] # not valid
    then
      echo
      echo "INFO ($scriptName): - the option selected '$optionSelected' is not valid"
      echo "     valid options for this configuration are:$validOptionList"
      echo 
      enterOneToContinue
    else
      setOptionValues
      minimumNumberOfDisksMustExistInAdsmvg=$((($firstJbod4packForEc - 1) * 4 ))
      numberOfDisksInAdsmVg=`lsvg -p adsmvg | grep hdisk | wc -l | awk '{print $1}'`
      if (( $minimumNumberOfDisksMustExistInAdsmvg > $numberOfDisksInAdsmVg ))
      then
        echo
        echo "NSM-95420-E: WARNING ($scriptName): the options selected: '$optionSelected'"
        echo "     is not valid. It requires that a minimum of '$minimumNumberOfDisksMustExistInAdsmvg' hdisks"
        echo "     exist in adsmvg prior to adding disks. The current number disk in"
        echo "     adsmvg is '$numberOfDisksInAdsmVg'" 
        echo 
        enterOneToContinue
        promptForFinalVerification='no'
      else
        promptForFinalVerification='yes'
      fi
    fi

  } # end verifyUserSelection

  promptFor16DiskSystem ()
  {
   
    validOptionList=''

    displayHeaderLines
    
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to8";  displayLine
    validOptionList="$validOptionList 1"
    
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to16";  displayLine
    validOptionList="$validOptionList 2"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op8to16";  displayLine
    validOptionList="$validOptionList 5"
 
    lineToPrint="$sepSolid";   displayLine # print trailer line

    promptForOptionSelection

    verifyUserSelection

  } # end promptFor16DiskSystem

  promptFor32DiskSystem ()
  {
   
    validOptionList=''

    displayHeaderLines
    
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to32";  displayLine
    validOptionList="$validOptionList 3"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op8to32";  displayLine
    validOptionList="$validOptionList 6"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op16to32";  displayLine
    validOptionList="$validOptionList 8"
 
    lineToPrint="$sepSolid";   displayLine # print trailer line

    promptForOptionSelection

    verifyUserSelection

  } # end promptFor32DiskSystem

  promptFor48DiskSystem ()
  {
   
    validOptionList=''

    displayHeaderLines
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to48";  displayLine
    validOptionList="$validOptionList 4"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op8to48";  displayLine
    validOptionList="$validOptionList 7"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op16to48";  displayLine
    validOptionList="$validOptionList 9"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op32to48";  displayLine
    validOptionList="$validOptionList 10"
 
    lineToPrint="$sepSolid";   displayLine # print trailer line

    promptForOptionSelection

    verifyUserSelection

  } # end promptFor48DiskSystem

  promptForAllDiskSystem () # not used for reference only
  {
   
    validOptionList=''

    displayHeaderLines
    
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to8";  displayLine
    validOptionList="$validOptionList 1"
    
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to16";  displayLine
    validOptionList="$validOptionList 2"
    
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to32";  displayLine
    validOptionList="$validOptionList 3"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op4to48";  displayLine
    validOptionList="$validOptionList 4"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op8to16";  displayLine
    validOptionList="$validOptionList 5"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op8to32";  displayLine
    validOptionList="$validOptionList 6"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op8to48";  displayLine
    validOptionList="$validOptionList 7"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op16to32";  displayLine
    validOptionList="$validOptionList 8"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op16to48";  displayLine
    validOptionList="$validOptionList 9"
  
    lineToPrint="$sepWplus"; displayLine
    lineToPrint="$op32to48";  displayLine
    validOptionList="$validOptionList 10"
 
    validOptionList="$validOptionList "
    lineToPrint="$sepSolid";   displayLine # print trailer line

    promptForOptionSelection

    verifyUserSelection

  } # end promptForAllDiskSystem

  # -------------------------------------------------------------------------------
  # Main prompt procedure
  # -------------------------------------------------------------------------------

  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="displayDiskAdditionPrompt"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  totalNumberOfDisksInSystem=`lsdev -Cc  pdisk | wc -l | awk '{print $1}'`

  # -------------------------------------------------------------------------------
  # Initialize Header Constants
  # -------------------------------------------------------------------------------
  mainHeader='|                 Option Table for Adding Disks                     |'
   opHeader1='|       |          | Num   |        |         | FIRST    | SECOND   |'
   opHeader2='|       |          | Disks | FC3635 | FC 3639 | FC 3533  | FC 3533  |'
   opHeader3='|Option | From-To  | Added |(4 to 8)|(8 to 16)|(16 to 32)|(32 to 48)|'
    sepWplus='|-------+----------+-------+--------+---------+----------+----------|'
    sepSolid='|-------------------------------------------------------------------|'
    op4to8='|   1   |  4 to  8 |   4   |  ADD   |         |          |          |'
     op4to16='|   2   |  4 to 16 |  12   |  ADD   |   ADD   |          |          |'
     op4to32='|   3   |  4 to 32 |  28   |  ADD   |   ADD   |   ADD    |          |'
     op4to48='|   4   |  4 to 48 |  44   |  ADD   |   ADD   |   ADD    |   ADD    |'
     op8to16='|   5   |  8 to 16 |   8   |        |   ADD   |          |          |'
     op8to32='|   6   |  8 to 32 |  24   |        |   ADD   |   ADD    |          |'
     op8to48='|   7   |  8 to 48 |  40   |        |   ADD   |   ADD    |   ADD    |'
    op16to32='|   8   | 16 to 32 |  16   |        |         |   ADD    |          |'
    op16to48='|   9   | 16 to 48 |  32   |        |         |   ADD    |   ADD    |'
    op32to48='|  10   | 32 to 48 |  16   |        |         |          |   ADD    |'

  # ---------------------------------------------------------
  # set up loop to prompt for the disk environment
  # ---------------------------------------------------------
  diskEnvironmentSelectionComplete='no'
  loopCounter=0
  while [ "$diskEnvironmentSelectionComplete" = 'no' ]
  do
    loopCounter=$(($loopCounter + 1))

    # -------------------------------------------------------------------
    # Display appropriatte header lines
    # -------------------------------------------------------------------
    case $totalNumberOfDisksInSystem in

      # -------------------------------------------------------------------
      # Display option lines for system with 16 disks
      # -------------------------------------------------------------------
      16) promptFor16DiskSystem
          : ;;

      # -------------------------------------------------------------------
      # Display option lines for system with 32 disks
      # -------------------------------------------------------------------
      32) promptFor32DiskSystem
          : ;;

      # -------------------------------------------------------------------
      # Display option lines for system with 48 disks
      # -------------------------------------------------------------------
      48) promptFor48DiskSystem
          : ;;

      # -------------------------------------------------------------------
      # Report error that system has an invalid number of disks
      # -------------------------------------------------------------------
      (*)
        echo
        echo "NSM-95421-E: ERROR ($scriptName): number of disks in the system '$totalNumberOfDisksInSystem' is not valid"
        echo "  The valid number of disks to have in a system is 16, 32 or 48"
        echo 
        echo "Exiting ...."
        echo
        enterOneToExit
        : ;;

    esac

    # ----------------------------------------------------------
    # Prompt for final verification
    # ----------------------------------------------------------
    if [ "$promptForFinalVerification" = 'yes' ]
    then
      echo 
      echo "Confirm the following settings: "
      echo
      echo "  The number of disks in the system prior to the MES was: $diskInSystemBeforeMes"
      echo "  The number of disks added to the system with this MES is: $diskAddedWithThisMes"
      echo
      echo "Is this correct? "
      echo
      sleep 1 # seems to be required, or output gets messed up
      PS3="Enter choice: "
      select correct in \
        'yes             ' \
        'no             '
      do
        case $REPLY in
          1) diskEnvironmentSelectionComplete="yes" ;;
          2) diskEnvironmentSelectionComplete="no" ;;
          *) echo 'Invalid Selection.' ;;
        esac
        if [[ -n $correct ]] ; then
           break
        fi
      done # select
    fi

  done # while [ "$diskEnvironmentSelectionComplete" = 'no' ]

  thisSubroutine="displayDiskAdditionPrompt"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end displayDiskAdditionPrompt

initializeJbod4Packs ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="initializeJbod4Packs"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"
 
  # -------------------------------------------------------------
  # Validate that the A and B loops are valid
  # -------------------------------------------------------------
  . createOrderedLoopLists
  createOrderedLoopListsRc=$?
  if [ $createOrderedLoopListsRc != 0 ]
  then
    echo "NSM-95422-E: ERROR - ($scriptName) -  Creating ordered loop lists."
    echo "  Fix problem and rerun"
    echo
    echo "Exiting ..."
    echo
    enterOneToExit
  fi
 
  # -------------------------------------------------------------
  # build the jbod4packs
  # -------------------------------------------------------------
  . buildJbod4packsForCxx

  thisSubroutine="initializeJbod4Packs"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end initializeJbod4Packs
 
verifySsaCabling ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="verifySsaCabling"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # --------------------------------------------------------------
  # Run automatic verication for disk plugging and ssa cabling
  # --------------------------------------------------------------
  autoVerifyCoralDiskInstallation
  autoVerifyCoralDiskInstallationRc=$?
  if [ $autoVerifyCoralDiskInstallationRc != 0 ]
  then
    echo
    echo "NSM-95423-E: ERROR - ($scriptName) - automatic verification of disk plugging and cabling failed"
    echo "       correct problem."
    echo
    echo "Exiting ..."
    echo
    enterOneToExit
  fi

  thisSubroutine="verifySsaCabling"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end verifySsaCabling

auditDisksFromReservedVgAsRequired ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="auditDisksFromReservedVgAsRequired"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"
  echo

   checkIfVgExists=$(lsvg  | grep reservedVg )
   if [ "$checkIfVgExists" = '' ]  # offline
   then
     echo "INFO: ($scriptName) - the reservedVg volume group does not exist"
     echo " this is valid if a previous execution of this script removed them"

   else
     # -----------------------------------------------------------------------
     # build hdisk list for each hdisk in reservedvg
     # -----------------------------------------------------------------------
     tempHdisksInReservedVg=`lspv | grep reservedVg | awk '{print $1}'`
     for currentHdisk in $tempHdisksInReservedVg
     do
       hdisksInReservedVg=" $hdisksInReservedVg $currentHdisk"
     done

     # -----------------------------------------------------------------------
     # process each disk that may need to be removed from reservedVg
     # ensure one resource, the associatted reserved logical volume, is the
     # only resource on the hdisk before removing it from the volume group
     # -----------------------------------------------------------------------
     highestRcFromCheckResourcesOnHdiskInReservedVg=0
     for currentHdisk in $hdisksInReservedVg
     do  
       checkResourcesOnHdiskInReservedVg $currentHdisk
       checkRc=$?
       if (( $checkRc == 0 ))
       then
         echo "  $currentHdisk in reservedVg is properly populated"
       else
         if (( $checkRc > $highestRcFromCheckResourcesOnHdiskInReservedVg ))
         then
           highestRcFromCheckResourcesOnHdiskInReservedVg=$checkRc
         fi
       fi
     done

     if (( $highestRcFromCheckResourcesOnHdiskInReservedVg == 1 ))
     then
       echo 
       echo "NSM-95446-W: WARNING ($scriptName) - One or more hdisks in reservedVg"
       echo "  were empty. Other audits for reservedVg did not fail. The script"
       echo "  can proceed with this condition(s)." 
       echo 
       enterOneToContinue

     elif (( $highestRcFromCheckResourcesOnHdiskInReservedVg == 2 ))
     then
       echo
       echo "NSM-95407-E: ERROR - ($scriptName) - audits failed for one or more hdisks in"
       echo "       the reservedVg volume group.  Error message(s) NSM-95406-E were"
       echo "       issued previously itemizing the failing condition(s)"
       echo
       echo "  Corrective action required. Exiting ..."
       echo
       enterOneToExit
     fi
  fi

  thisSubroutine="auditDisksFromReservedVgAsRequired"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # auditDisksFromReservedVgAsRequired

removeDisksFromReservedVgAsRequired ()
{

  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="removeDisksFromReservedVgAsRequired"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # ---------------------------------------------------------------
  # Build list of disks that may need to be removed from reservedVg
  # ---------------------------------------------------------------
  if (( $firstJbod4packForEc == 2 ))
  then
    if (( $lastJbod4packForEc >= 4 ))
    then
      hdiskIdsToRemove="$jbod4pack2 $jbod4pack3 $jbod4pack4"
    elif (( $lastJbod4packForEc == 2 ))
    then
      hdiskIdsToRemove="$jbod4pack2"
    fi

  elif (( $firstJbod4packForEc == 3 )) && (( $lastJbod4packForEc >= 4 ))
  then
    hdiskIdsToRemove="$jbod4pack3 $jbod4pack4"

  else
    hdiskIdsToRemove=''
  fi

  echo
  if  [ "$hdiskIdsToRemove" = '' ]
  then
    echo "INFO ($scriptName) - None of this disks that are being added would ever reside"
    echo "     in the reservedVg volume group -OR- they have already been removed"
  else
     # -----------------------------------------------------------------------
     # Ensure the reservedVg is varied online
     # -----------------------------------------------------------------------
     echo
     checkIfVgOnlineAndActive=$(lsvg -o | grep reservedVg )
     if [ "$checkIfVgOnlineAndActive" != '' ]  # online
     then
       echo "INFO: ($scriptName) - the reservedVg volume group is online"
       echo "  Checking to see if hdisks must be removed"
       echo 

       # -----------------------------------------------------------------------
       # build hdisk list for each hdisk in reservedvg
       # -----------------------------------------------------------------------
       tempHdisksInReservedVg=`lspv | grep reservedVg | awk '{print $1}'`
       for currentHdisk in $tempHdisksInReservedVg
       do
         hdisksInReservedVg=" $hdisksInReservedVg $currentHdisk"
       done

       # -----------------------------------------------------------------------
       # process each disk that may need to be removed from reservedVg
       # ensure one resource, the associatted reserved logical volume, is the
       # only resource on the hdisk before removing it from the volume group
       # -----------------------------------------------------------------------
       oneOrMoreValidationErrorsForDisksInReservedVg='no'
       for curHdiskID in $hdiskIdsToRemove
       do  
         currentHdisk="hdisk$curHdiskID"
         echo 
         echo "Determining if '$currentHdisk' must be removed from the reservedVg volume group"
         echo
         checkIfHdiskInReservedVg=`compareLists match $currentHdisk ! $hdisksInReservedVg`
         if [ "$checkIfHdiskInReservedVg" = '' ]
         then
           echo " -- $currentHdisk is not in reservedVg, this is likely caused by this"
           echo "     script previously removing it" 
         else
           echo " -- $currentHdisk is in reservedVg"

           checkResourcesOnHdiskInReservedVg $currentHdisk
           checkRc=$?
           if (( $checkRc <= 1 ))   
           then
             theCommand="reducevg '-d' '-f' reservedVg $currentHdisk"
             echo '    Issuing command: "'$theCommand'"'
             eval $theCommand
             theCommandRc=$?
             if (( $theCommandRc != 0 ))
             then
               echo 
               echo "NSM-95425-E: ERROR ($scriptName) - failure removing $currentHdisk from reservedVg"
               oneOrMoreValidationErrorsForDisksInReservedVg='yes'
             fi
           else
             oneOrMoreValidationErrorsForDisksInReservedVg='yes'
           fi
         fi
       done

       if [ "$oneOrMoreValidationErrorsForDisksInReservedVg" = 'yes' ]
       then
         echo
         echo "Exiting ..."
         echo
         enterOneToExit
       fi

     # ---------------------------------------------------------
     # the reservedVg volume group is not online
     # ---------------------------------------------------------
     else # offline or does not exist

       checkIfVgExistsButOffline=$(lsvg  | grep reservedVg )
       if [ "$checkIfVgExistsButOffline" != '' ]  # offline
       then
         echo
         echo "NSM-95426-E: ERROR - ($scriptName) -  the reservedVg volume group is varried off line"
         echo "  Corrective action required. This script cannot proceed until this"
         echo "  volume group is varied online"
         echo 
         echo "Exiting ..."
         echo 
         exit 1

       else
         echo "INFO: ($scriptName) - the reservedVg volume group does not exist"
         echo " this is valid if a previous execution of this script removed them"
       fi
     fi
  fi

  thisSubroutine="removeDisksFromReservedVgAsRequired"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end removeDisksFromReservedVgAsRequired
 
check3466DiskResourcesAixViewOnly ()
{

  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="check3466DiskResourcesAixViewOnly"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # ------------------------------------------------
  # make sure the loops are intact
  # ------------------------------------------------
  errorDetected='no'
  echo
  echo "INFO ($scriptName) - checking to ensure both A and B loops are intact"
  vdisk -c -l >/dev/null 2>&1
  theRc=$?

  if (( $theRc == 0 ))
  then
    echo "INFO ($scriptName) - both the A and B loops are intact"
  else
    errorDetected='yes'
    echo "NSM-95427-E: ERROR ($scriptName) - there is an open Loop"
    echo
    theCommand='vdisk -c -l'
    echo "Issuing command: '$theCommand' to show diagnostic information"
    echo
    eval $theCommand
    echo
  fi

  # ------------------------------------------------
  # make sure the pdisk/hdisks have the proper number
  # relationship
  # ------------------------------------------------
  echo; echo "$stdSeperatorLine"
  echo
  echo "INFO ($scriptName) - checking to ensure that the numeric portion of each"
  echo "      pdisk  ID is exactly 2 less than the numeric portion of the hdisk ID"
  vdisk -c -p >/dev/null 2>&1
  theRc=$?

  if (( $theRc == 0 ))
  then
    echo "INFO ($scriptName) - the numeric pdisk/hdisk relationship is correct"
  else
    errorDetected='yes'
    echo "NSM-95428-E: ERROR ($scriptName) - the numeric pdisk/hdisk relationship is not correct"
    echo
    theCommand='vdisk -c -p'
    echo "Issuing command: '$theCommand' to show diagnostic information"
    echo
    eval $theCommand
    echo
  fi

  # ------------------------------------------------
  # make sure the name of TSM resources match the 
  # hdisk they are on
  # ------------------------------------------------
  echo; echo "$stdSeperatorLine"
  echo
  echo "INFO ($scriptName) - checking to ensure that the hdisk identifer in the"
  echo "      name of each TSM resource matches the hdisk ID on which it resides"
  # vdisk -c -m >/dev/null 2>&1
  cd /tmp
  vdisk -c -m >/dev/null 2>&1
  cd -
  theRc=$?
  echo "theRc $theRc"

  if (( $theRc == 0 ))
  then
    echo "INFO ($scriptName) - the hdisk ID for each TSM resource matches the hdisk it is on"
  else
    errorDetected='yes'
    echo "NSM-95429-E: ERROR ($scriptName) - the hdisk ID for at least one TSM resource does not"
    echo "      match the hdisk it is on"
    echo
    theCommand='vdisk -c -m'
    echo "Issuing command: '$theCommand' to show diagnostic information"
    echo
    eval $theCommand
    echo
  fi

  # ------------------------------------------------
  # Display final error message if one or more 
  # errors ocurred
  # ------------------------------------------------
  if [ "$errorDetected" = 'yes' ]
  then
    echo
    echo "NSM-95430-E: ERROR ($scriptName) - one or more errors ocurred"
    echo "      Corrective action required"
    echo
    echo "Exiting ..."
    echo
    enterOneToExit
  fi

  thisSubroutine="check3466DiskResourcesAixViewOnly"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end check3466DiskResourcesAixViewOnly

commonVerification ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="commonVerification"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # -------------------------------------------------------------
  # Initialize stuff
  # -------------------------------------------------------------
  initializeStuff

  # -------------------------------------------------------------
  # Check that filesets in installation directory are at or
  # higher than the required level
  # -------------------------------------------------------------
  if [ "$skipCheckFilesetUpdateLevels" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
    checkFilesetUpdateLevels
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - checking fileset levels (skipCheckFilesetUpdateLevels=$skipCheckFilesetUpdateLevels , skipVerificationSteps=$skipVerificationSteps)"
  fi

  # -------------------------------------------------------------
  # Verify the operating system is at least 4.3.3
  # -------------------------------------------------------------
  if [ "$skipVerifyOperatingSystemLevel" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
   verifyOperatingSystemLevel
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - skip verfiying operating system level (skipVerifyOperatingSystemLevel=$skipVerifyOperatingSystemLevel , skipVerificationSteps=$skipVerificationSteps)"
  fi
 
  # -------------------------------------------------------------
  # Set the type of pdisk: must be 18GB JBOD disks
  # -------------------------------------------------------------
  setPdiskType

  # ----------------------------------------------------------------------
  # Find out what type of system this is. Must be an H70
  # ----------------------------------------------------------------------
  if [ "$skipVerifySystemIsAnH70" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
    verifySystemIsAnH70
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - skip H70 verfication test (skipVerifySystemIsAnH70=$skipVerifySystemIsAnH70 , skipVerificationSteps=$skipVerificationSteps)"
  fi

  # -------------------------------------------------------------
  # Verify volume group existent and content
  # -------------------------------------------------------------
  if [ "$skipSanityCheckVolumeGroups" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
    sanityCheckVolumeGroups
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - (skipCheckFilesetUpdateLevels=$skipCheckFilesetUpdateLevels , skipVerificationSteps=$skipVerificationSteps)"
  fi

  # -------------------------------------------------------------
  # Verify volume group existent and content
  # -------------------------------------------------------------
  if [ "$skipListVolumeGroups" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
    listVolumeGroups
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - (skipListVolumeGroups=$skipListVolumeGroups , skipVerificationSteps=$skipVerificationSteps)"
  fi

  # -------------------------------------------------------------
  # Verify contents of reservedVg
  # -------------------------------------------------------------
  if [ "$skipAuditDisksFromReservedVgAsRequired" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
   auditDisksFromReservedVgAsRequired
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - auditingDisks in reservedVg ($skipAuditDisksFromReservedVgAsRequired=$skipAuditDisksFromReservedVgAsRequired , skipVerificationSteps=$skipVerificationSteps)"
  fi

  # -----------------------------------------------------------------------
  # verify communication with the server
  # -----------------------------------------------------------------------
  verifyCommunicationWithTsmServer

  # -------------------------------------------------------------
  #  Over-rides  Required for compatibility with scripts that 
  # are called
  # -------------------------------------------------------------
  export adsmID=$adminUserID
  export adsmPSWD=$adminPassword
  export adsmServer=$serverStanza

  export adminId=$adminUserID
  export adminPass=$adminPassword

  export netStoreMachineType='C01'
  export numDisk=$totalNumberOfDisksInSystem

  # -----------------------------------------------------------------------
  # Verify Basic TSM stuff
  # -----------------------------------------------------------------------
  if [ "$skipVerifyTsmStuff" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
    verifyTsmStuff
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - verifying TSM environment (skipVerifyTsmStuff=$skipVerifyTsmStuff , skipVerificationSteps=$skipVerificationSteps)"
  fi

  # -------------------------------------------------------------
  # Initialize "jbod4pack" environment variables
  # -------------------------------------------------------------
  initializeJbod4Packs

  # -------------------------------------------------------------
  # Verify the SSA drawers are properly cabled
  # -------------------------------------------------------------
  if [ "$skipVerifySsaCabling" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
    verifySsaCabling
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - skip checking ssa cabling (skipVerifySsaCabling=$skipVerifySsaCabling , skipVerificationSteps=$skipVerificationSteps)"
  fi

  # -------------------------------------------------------------
  # Verify validity of disk resources
  # -------------------------------------------------------------
  skipCheck3466DiskResourcesAixViewOnly='yes'
  if [ "$skipCheck3466DiskResourcesAixViewOnly" = 'no' ] && [ "$skipVerificationSteps" = 'no' ]
  then
    check3466DiskResourcesAixViewOnly
  else
    echo; echo "$stdSeperatorLine"
    echo "INFO ($scriptName) - checkin 3466 resources (skipCheck3466DiskResourcesAixViewOnly=$skipCheck3466DiskResourcesAixViewOnly , skipVerificationSteps=$skipVerificationSteps)"
  fi

  thisSubroutine="commonVerification"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end commonVerification

buildHdiskAddList ()
{
  # ----------------------------------------------
  # Loop through 4 packs and init hdisk ID list
  # ----------------------------------------------
  fourPackCounter=$firstJbod4packForEc
  listOfHdiskIDsToAdd=''
  while (( $fourPackCounter <= $lastJbod4packForEc ))
  do
    jbod4PackName="jbod4pack$fourPackCounter"
    hdiskIdListForCurrent4pack=`eval echo '$'$jbod4PackName`
    listOfHdiskIDsToAdd="$listOfHdiskIDsToAdd $hdiskIdListForCurrent4pack"

    fourPackCounter=$(($fourPackCounter + 1))
  done

  # ----------------------------------------------
  # Use ID list to build hdisk list
  # ----------------------------------------------
  returnHdiskListToAdd=''
  for currentEntry in $listOfHdiskIDsToAdd
  do
    returnHdiskListToAdd="$returnHdiskListToAdd hdisk$currentEntry"
  done

  # ----------------------------------------------
  # return the hdisk list via STDOUT
  # ----------------------------------------------
  echo "$returnHdiskListToAdd"

} # end buildHdiskAddList

phase1GetReady ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="phase1GetReady"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # ------------------------------------------
  # Install software updates
  # ------------------------------------------
  installUpdates

  # -------------------------------------------------
  # Invoke fixSSA to check if pdisks Hdisks are ordered correctly
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  echo
  fixSSA -q 
  fixSsaRc=$?
  echo
  if (( $fixSsaRc == 0 ))
  then
    echo "INFO ($scriptName) - hdisks / pdisks are named correctly" 
  else
    echo "NSM-95404-E: ERROR ($scriptName) - hdisks / pdisks are NOT named correctly" 
    echo 
    echo "  This can be a serious problem! Contact IBM support for assistance"
    echo "  Typically the fixSSA routine should be called to resolve the problem"
    echo "  However, we can not be certain how long the hdisks/pdisks IDs have"
    echo "  been improperly named. Before running fixSSA each disk in the"
    echo "  adsmvg, adsmvg2 and reservedVg volume groups should be inventoried"
    echo "  to check their contents. If the contents do not match the hdisk"
    echo "  on which they reside, fixSSA may fix the problem. If the contents"
    echo "  do match the hdisk they are on more extensive diagnosis is required"
    echo 
    echo "  The steps for running fixSSA are:"
    echo 
    echo "  1) endadsm"
    echo "  2) prebackup"
    echo "  3) fixSSA"
    echo "  4) postbackup"
    echo "  5) Inventory disk contents"
    echo "  6) startadsm if disk contents are valid"
    echo 
    echo "  Exiting ...."
    echo 
    enterOneToExit
  fi 

  # ------------------------------------------
  # Initiate common verification
  # ------------------------------------------
  userSelectedPhase='phase 1'
  commonVerification

  # -------------------------------------------------------------
  # Make user enter a response to continue with revfile
  # -------------------------------------------------------------
  echo
  echo "INFO ($scriptName) - Phase 1 is SUCCESSFULL (`date`)"
  echo
  enterOneToContinue

  thisSubroutine="phase1GetReady"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
} # end phase1GetReady
 
 
phase2UpdateSystem  ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="phase2UpdateSystem"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # -------------------------------------------------------------
  # Initiate common verification
  # -------------------------------------------------------------
  userSelectedPhase='phase 2'
  commonVerification
 
  # -------------------------------------------------------------
  # Reorder the SSA pdisks if required
  # -------------------------------------------------------------
  if [ "$skipReorderSsaPdisksAndHdisks" = 'no' ]
  then
    reorderSsaPdisksAndHdisks
  else
    echo; echo "$stdSeperatorLine";echo
    echo "INFO ($scriptName) - skip re-ordering SSA disks (skipReorderSsaPdisksAndHdisks=$skipReorderSsaPdisksAndHdisks)"
  fi

  # ----------------------------------------------------------------------
  # Initialize input from pre-existing progress log if it exists
  # ----------------------------------------------------------------------
  if [ -f "$progressLog" ]
  then
    echo
    echo "NSM-95449-W: WARNING ($scriptName) - the progress log for this script exists."
    echo 
    echo "  The progress log file is: $progressLog"
    echo 
    echo "  The progress log contains user input from a previous execution of this"
    echo "  script. The information from the progress log will be used."
    echo

    promptInformationMustBeObtainedFromProgressLog='yes'

    other4packDBalloc=`cat $progressLog | grep 'other4packDBalloc=' | cut -f2 -d'='`
    diskInSystemBeforeMes=`cat $progressLog | grep 'diskInSystemBeforeMes=' | cut -f2 -d'='`
    diskAddedWithThisMes=`cat $progressLog | grep 'diskAddedWithThisMes=' | cut -f2 -d'='`
    firstJbod4packForEc=`cat $progressLog | grep 'firstJbod4packForEc=' | cut -f2 -d'='`
    lastJbod4packForEc=`cat $progressLog | grep 'lastJbod4packForEc=' | cut -f2 -d'='`
    tsmDefintionStarted=`cat $progressLog | grep 'tsmDefintionStarted=' | cut -f2 -d'='`
    aixDefintionStarted=`cat $progressLog | grep 'aixDefintionStarted=' | cut -f2 -d'='`

    echo "  The database allocation is $other4packDBalloc Megabytes"
    echo "  The number of disks active in the system prior to the MES was: $diskInSystemBeforeMes"
    echo "  The number of disks added with this MES is: $diskAddedWithThisMes"
    echo "  The first JBOD 4 Pack added by this MES is: $firstJbod4packForEc"
    echo "  The last JBOD 4 Pack added by this MES is: $lastJbod4packForEc"
    echo  
    echo "  If the above values are not correct the steps for starting over must"
    echo "  followed EXACTLY. Detailed information for starting over is provided"
    echo "  in an Appendix in the Install Instruction. IBM Support should be"
    echo "  contacted for assistance if there are any doubts regarding this procedure."

    echo
    if [ "$tsmDefintionStarted" != '' ]
    then
      echo "  The log indicates that the definition of new resources to TSM was started."
      echo "  TSM must be checked to determine if any new resources were added."
      echo "  All new resources must be removed."
    else
      echo "  The log indicates that the definition of new resources to TSM was NOT started."
    fi

    echo
    if [ "$aixDefintionStarted" != '' ]
    then
      echo "  The log indicates that the creation of the AIX resources has started."
      echo "  AIX must be checked to determine if any new resources were added."
      echo "  All new resources must be removed."
    else
      echo "  The log indicates that the creation of the AIX resources has NOT started."
      echo "  However, logical volumes for new TSM database volumes may still have"
      echo "  been added. These new logical volumes must be removed,"
    fi

    echo
    echo "  After the above steps have been completed the progress log must be"
    echo "  deleted: 'rm $progressLog'"

    echo 
    enterOneToExit

    progressLogCreatedThisRun='no'
  else
    # ----------------------------------------------------------------------
    # Prompt user for input. Verify input is valid.
    # ----------------------------------------------------------------------
    displayDiskAdditionPrompt
  fi

  # -----------------------------------------------------
  # set variable to control last jbod4pack used
  # this is an override that is used by other scripts
  # that are called
  # -----------------------------------------------------
  export jbod4packs="$lastJbod4packForEc"
 
  # ----------------------------------------------------------------------
  # Build jbod4packs again since some jbod4packs must be skipped
  # (actually should only change if the lastJbod4pack=2)
  # Left in so dev can easily override
  # ----------------------------------------------------------------------
  . buildJbod4packsForCxx

  # -------------------------------------------------------------
  # Calculate the last jbod 4 pack to add
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine"
  echo
  echo "The first jbod4pack to add is: $firstJbod4packForEc"
  echo "The last jbod4pack to add is: $lastJbod4packForEc"
  echo

  # -------------------------------------------------------------
  # Remove hdisks from reserveVg volume group if required
  # -------------------------------------------------------------
  removeDisksFromReservedVgAsRequired

  # -------------------------------------------------------------
  # Extend the ADSM volume group as appropriatte
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine";echo
  if [ "$skipExtendAdsmVg" = "no" ]
  then
    if [ "$firstJbod4packForEc" -le 8 ]
    then
       extendADSMvg.stsm $adsmvg
       rc=$?
       if [ "$rc" != 0 ]
       then
         echo "NSM-95431-E: ERROR ($scriptName) - Could not extend ADSMvg."
         echo
         echo "Exiting ..."
         echo
         enterOneToExit
       fi
    fi
 
    if [ "$lastJbod4packForEc" -ge 9 ]
    then
       adsmvgExists=`lsvg $adsmvg2`
       if [ "$adsmvgExists" = '' ]
       then
          createADSMvg.stsm $adsmvg2
       else
          extendADSMvg.stsm $adsmvg2
       fi
       rc=$?
       if [ "$rc" != 0 ]
       then
         echo "NSM-95432-E: ERROR - ($scriptName) - Can't create $adsmvg2 volume group. Fix problem and rerun."
         echo
         echo "Exiting ..."
         echo
         enterOneToExit
       fi
    fi
  else
    echo "INFO ($scriptName) - skip extending adsmvg and/or adsmvg2 volume groups (skipExtendAdsmVg=$skipExtendAdsmVg)"
  fi
 
  # -------------------------------------------------------------
  # Prompt for the amount of space to use for DB volumes
  # ------------------------------------------------------------
  theListOfHdisksToAdd=''
  fourPackCounter=$firstJbod4packForEc
  fourPackCounter=1
  while (( $fourPackCounter <= $lastJbod4packForEc ))
  do
    jbod4PackName="jbod4pack$fourPackCounter"
    hdiskIdListForCurrent4pack=`eval echo '$'$jbod4PackName`
    if (( $fourPackCounter < $firstJbod4packForEc ))
    then 
      : 
    else
      for currentID in $hdiskIdListForCurrent4pack
      do
        theListOfHdisksToAdd="$theListOfHdisksToAdd hdisk$currentID"
      done
    fi
    fourPackCounter=$(($fourPackCounter + 1))
  done

  # -------------------------------------------------------------
  # Prompt for the amount of space to use for DB volumes
  # ------------------------------------------------------------
  promptForAmountOfNewDbSpace  adsmvg $theListOfHdisksToAdd

  # -------------------------------------------------------------
  # Create the resource lists
  # -------------------------------------------------------------

  # --------------------------------------------------------------
  # Bummer -- only 32 physical volumes in a vg, so split into 2
  # vgs if 3 fully populated drawers or more.
  # --------------------------------------------------------------
  echo; echo "$stdSeperatorLine"
  export firstJbod4packForEc
  export lastJbod4packForEc
  if [ "$lastJbod4packForEc" -gt 8 ]
  then
    export firstJbod4packForceAdsmvgDb=$firstJbod4packForEc
    export firstJbod4packForceAdsmvgBf=$firstJbod4packForEc
    if [ $firstJbod4packForEc -le 8 ]
    then
      # --------------------------------------------------------------
      # This is the synch point for DB selection. If the script
      # is rerun after a failure the following values must be 
      # read from disk to continue
      # --------------------------------------------------------------
      if [ "$progressLogCreatedThisRun" = 'no' ]
      then
        : # skip creating progress log
      else
        echo "other4packDBalloc=$other4packDBalloc" >> $progressLog
        echo "diskInSystemBeforeMes=$diskInSystemBeforeMes" >> $progressLog
        echo "diskAddedWithThisMes=$diskAddedWithThisMes" >> $progressLog
        echo "firstJbod4packForEc=$firstJbod4packForEc" >> $progressLog
        echo "lastJbod4packForEc=$lastJbod4packForEc" >> $progressLog
      fi
      export lastJbod4packForceAdsmvgBf=8
      lgList1='' # never add log space with an MES
      dbList1=`dynamicGetADSMresource db all  $firstJbod4packForceAdsmvgDb $firstJbod4packForceAdsmvgDb x`
      bfList1=`eval dynamicGetADSMresource bf all  $firstJbod4packForceAdsmvgBf $lastJbod4packForceAdsmvgBf x`

      export firstJbod4packForceAdsmvg2Db=9
      export lastJbod4packForceAdsmvg2Db=9
      export firstJbod4packForceAdsmvg2Bf=9
      export lastJbod4packForceAdsmvg2Bf=$lastJbod4packForEc
 
      lgList2='' # never add log space with an MES
      dbList2='' # db space already added, if required, on disk in adsmvg
      bfList2=`eval dynamicGetADSMresource bf all  $firstJbod4packForceAdsmvg2Bf $lastJbod4packForceAdsmvg2Bf x`

    else # adding disks starting with disk 32
      # --------------------------------------------------------------
      # This is the synch point for DB selection. If the script
      # is rerun after a failure the following values must be 
      # read from disk to continue
      # --------------------------------------------------------------
      if [ "$progressLogCreatedThisRun" = 'no' ]
      then
        : # skip creating progress log
      else
        echo "other4packDBalloc=$other4packDBalloc" >> $progressLog
        echo "diskInSystemBeforeMes=$diskInSystemBeforeMes" >> $progressLog
        echo "diskAddedWithThisMes=$diskAddedWithThisMes" >> $progressLog
        echo "firstJbod4packForEc=$firstJbod4packForEc" >> $progressLog
        echo "lastJbod4packForEc=$lastJbod4packForEc" >> $progressLog
      fi
      export firstJbod4packForceAdsmvg2Db=$firstJbod4packForEc
      export lastJbod4packForceAdsmvg2Db=$firstJbod4packForceAdsmvg2Db
      export firstJbod4packForceAdsmvg2Bf=$firstJbod4packForEc
      export lastJbod4packForceAdsmvg2Bf=$lastJbod4packForEc
 
      lgList2='' # never add log space with an MES
      dbList2=`dynamicGetADSMresource db all  $firstJbod4packForceAdsmvg2Db $lastJbod4packForceAdsmvg2Db  x`
      bfList2=`eval dynamicGetADSMresource bf all  $firstJbod4packForceAdsmvg2Bf $lastJbod4packForceAdsmvg2Bf x`
    fi

  else # disks 32-48 are not present
    # --------------------------------------------------------------
    # This is the synch point for DB selection. If the script
    # is rerun after a failure the following values must be 
    # read from disk to continue
    # --------------------------------------------------------------
    if [ "$progressLogCreatedThisRun" = 'no' ]
    then
      : # skip creating progress log
    else
      echo "other4packDBalloc=$other4packDBalloc" >> $progressLog
      echo "diskInSystemBeforeMes=$diskInSystemBeforeMes" >> $progressLog
      echo "diskAddedWithThisMes=$diskAddedWithThisMes" >> $progressLog
      echo "firstJbod4packForEc=$firstJbod4packForEc" >> $progressLog
      echo "lastJbod4packForEc=$lastJbod4packForEc" >> $progressLog
    fi
    export firstJbod4packForceAdsmvgDb=$firstJbod4packForEc
    export firstJbod4packForceAdsmvgBf=$firstJbod4packForEc
    export lastJbod4packForceAdsmvgBf=$lastJbod4packForEc
    lgList1='' # never add log space with an MES
    dbList1=`dynamicGetADSMresource db all  $firstJbod4packForceAdsmvgDb $firstJbod4packForceAdsmvgDb x`
    bfList1=`eval dynamicGetADSMresource bf all  $firstJbod4packForceAdsmvgBf $lastJbod4packForceAdsmvgBf x`

    export firstJbod4packForceAdsmvg2Db=0
    export lastJbod4packForceAdsmvg2Db=0
    export firstJbod4packForceAdsmvg2Bf=0
    export lastJbod4packForceAdsmvg2Bf=0
    lgList2=''
    dbList2=''
    bfList2=''
  fi
 
  # ------------------------------------------------------------------------
  # log info for recources to be created in adsmvg volume group, if any
  # ------------------------------------------------------------------------
  echo; echo "$stdSeperatorLine"
  if [ "$firstJbod4packForEc" -le 8 ]
  then
    echo "The log list1 (`echo $lgList1 | wc -w | awk '{print $1}'`) is: $lgList1"
    echo "The DB list1 (`echo $dbList1 | wc -w | awk '{print $1}'`) is: $dbList1"
    echo "The primary bitfile list1 (`echo $bfList1 | wc -w | awk '{print $1}'`) is: $bfList1"
  fi
 
  # ------------------------------------------------------------------------
  # log info for recources to be created in adsmvg2 volume group, if any
  # ------------------------------------------------------------------------
  if [ "$lastJbod4packForEc" -ge 9 ]
  then
     echo "The log list2 (`echo $lgList2 | wc -w | awk '{print $1}'`) is: $lgList2"
     echo "The DB list2 (`echo $dbList2 | wc -w | awk '{print $1}'`) is: $dbList2"
     echo "The primary bitfile list2 (`echo $bfList2 | wc -w | awk '{print $1}'`) is: $bfList2"
  fi

  # -------------------------------------------------------------
  # Select disk storage pool to use if required
  # -------------------------------------------------------------
  selectDiskPoolToAddTo

  echo 
  echo "**********************************************************************"
  echo "*"
  echo "* INFO ($scriptName) - this script has collected all of the information"
  echo "*       required to proceed. The script should run to completion"
  echo "*       with no further interaction required."
  echo "*"
  echo "*   The remaining steps in this script perform the following functions:"
  echo "*"
  echo "*     1) Create the AIX logical volume and JFS filesystems for the new"
  echo "*        TSM resources"
  echo "*"
  echo "*     2) Issue 'dsmfmt' to format the new TSM resources"
  echo "*"
  echo "*     3) Define the new resources to TSM"
  echo "*"
  echo "*     4) Record information about the changes to 3466"
  echo "*"
  echo "*   The amount of time left to complete the script is dependent on "
  echo "*   several factors:"
  echo "*"
  echo "*     1) The number of disks being added"
  echo "*"
  echo "*     2) The amount of TSM data base space being added"
  echo "*"
  echo "*     3) Other activity that is taking place during the time the "
  echo "*        script is running."
  echo "*"
  echo "**********************************************************************"
  echo
  enterOneToContinue
  echo "**********************************************************************"
  echo "*"
  echo "*   The following estimates for time required to complete this"
  echo "*   are based on observations with no activity. The low estimate"
  echo "*   reflects that no new TSM DB volumes are added. The high estimate"
  echo "*   reflects that one disk is allocated to a new Primary DB volume."
  echo "*"
  echo "*   The number of disks being added with this MES is: '$numberOfHdisksToAdd'"
  echo "*"
  echo "*     --  4 disks:  30 to  40 minutes"
  echo "*     --  8 disks:  60 to  70 minutes"
  echo "*     -- 12 disks:  90 to 100 minutes"
  echo "*     -- 16 disks: 120 to 130 minutes"
  echo "*     -- 20 disks: 150 to 160 minutes"
  echo "*     -- 24 disks: 180 to 190 minutes"
  echo "*     -- 28 disks: 210 to 220 minutes"
  echo "*     -- 32 disks: 240 to 250 minutes"
  echo "*     -- 36 disks: 270 to 280 minutes"
  echo "*     -- 40 disks: 300 to 310 minutes"
  echo "*     -- 44 disks: 330 to 340 minutes"
  echo "*"
  echo "**********************************************************************"
  echo
  enterOneToContinue

  # -------------------------------------------------------------
  # Record that the AIX resource definition phase has started
  # -------------------------------------------------------------
  echo "aixDefintionStarted=yes"  >> $progressLog

  # -------------------------------------------------------------
  # Issue a time stamp so the amount of time to comlete the 
  # unattended phase can be calculated
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine";echo
  echo 
  echo "INFO: ($scriptName) - timestamp after unattended prompt: `date`"
 
  # -------------------------------------------------------------
  # Create the journaled filesystems for resources to be added
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine";echo
  if [ "$skipCreateJFS" = "no" ]
  then
    if [ "$firstJbod4packForEc" -le 8 ]
    then
       createJFS.stsm "$adsmvg" "$dbList1" "$lgList1" "$bfList1"
       rc=$?
       if [ "$rc" != 0 ]
       then
         echo "NSM-95433-E: ERROR - ($scriptName) - Can't create JFS.  Fix problem and rerun phase."
         echo
         echo "Exiting ..."
         echo
         enterOneToExit
       fi
    fi
 
    if [ "$lastJbod4packForEc" -gt 8 ]
    then
       createJFS.stsm "$adsmvg2" "$dbList2" "$lgList2" "$bfList2"
       rc=$?
       if [ "$rc" != 0 ]
       then
         echo "NSM-95433-E: ERROR - ($scriptName) - Can't create JFS (for 3rd drawer).  Fix problem and rerun phase."
         echo
         echo "Exiting ..."
         echo
         enterOneToExit
       fi
    fi
  else
    echo; echo "$stdSeperatorLine"
    echo "INFO ($scriptName) skip creating ADSM JFSs for ADSM resources (skipCreateJFS=$skipCreateJFS)"
  fi
 
  # -------------------------------------------------------------
  # Create the ADSM resources
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine";echo
  if [ "$skipCreateADSMresource" = "no" ]
  then
    createADSMresource.stsm "$dbList1" "$lgList1" "$bfList1"  "$dbList2" "$lgList2" "$bfList2"
    rc=$?
    if [ "$rc" != 0 ]
    then
      echo "NSM-95434-E: ERROR ($scriptName) - Could not create ADSM resources."
      echo
      echo "Exiting ..."
      echo
      enterOneToExit
    fi
  else
    echo "INFO ($scriptName) - skip creating ADSM resources (skipCreateADSMresource=$skipCreateADSMresource)"
  fi

  # -------------------------------------------------------------
  # Record that the TSM resource definition phase has started
  # -------------------------------------------------------------
  echo "tsmDefintionStarted=yes"  >> $progressLog

  # -------------------------------------------------------------
  # Define space to adsm now
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine";echo
  if [ "$skipDefineADSM" = "no" ]
  then
    export firstDiskSet="$firstJbod4packForEc"
    defineADSMvol.stsm db
    rc=$?
    if [ "$rc" != 0 ]
    then
      echo "NSM-95435-E: ERROR ($scriptName) - Could not define db volumes."
      echo
      echo "Exiting ..."
      echo
      enterOneToExit
    fi
 
    defineADSMvol.stsm bf "$diskPoolToUse" 
    rc=$?
    if [ "$rc" != 0 ]
    then
      echo "NSM-95436-E: ERROR ($scriptName) - Could not define storage pool volumes."
      echo
      echo "Exiting ..."
      echo
      enterOneToExit
    fi
  else
    echo "skip: defining ADSM resources (skipDefineADSM=$skipDefineADSM)"
  fi
 
 
  # -------------------------------------------------------------
  # Save out system info
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine";echo
  if [ "$skipSaveHWinfo" = "no" ]
  then
     dir="/usr/nsm.3466/sysInfo"
     /usr/nsm.3466/script/saveSysInfo sysInfoaa2.out $adminId $adminPass ss-m
     echo "Mfg said this system had $diskNum disks." >> $dir/sysInfoaa2.out
     echo "Mfg said this system had $numLib libraries." >> $dir/sysInfoaa2.out
     echo "Mfg said this system had $numDrivePerLib drives in library 1." >> $dir/sysInfoaa2.out
     echo "Mfg said this system had $numDrivePerLib2 drives in library 2." >> $dir/sysInfoaa2.out
 
     # this records off disk info needed for replace.ssa
     echo "Issuing 'record.ssa all'"
     /usr/nsm.3466/script/record.ssa all
 
     # this records off disk info needed for record.build.script
     echo "Issuing 'record 1 $adminId $adminPass ss-m'"
     /usr/nsm.3466/script/record 1 $adminId $adminPass ss-m
 
     # this creates the record.build.script
     echo "Issuing 'record 2 $adminId $adminPass ss-m'"
     /usr/nsm.3466/script/record 2 $adminId $adminPass ss-m
  else
    echo "INFO ($scriptName) - skip saving hardware info (skipSaveHWinfo=$skipSaveHWinfo)"
  fi
 
  # -------------------------------------------------------------
  #  add auto back into the file
  # -------------------------------------------------------------
  mkitabCommand="mkitab 'autosrvr:2:once:/usr/nsm.3466/script/startadsm all'"
  echo; echo "$stdSeperatorLine"
  echo "INFO: ($scriptName) - adding 'addauto' back into inittab."
  echo
  echo "Issuing command: $mkitabCommand"
  echo
  eval $mkitabCommand
 
  # -------------------------------------------------------------
  # archive the progress log
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine"
  echo 
  echo "INFO ($scriptName) - archive $progressLog"
  echo 
  ls -l $progressLog

  echo 
  cat $progressLog

  echo 
  theCommand="mv $progressLog $progressLog""_done_`date +%m%d%Y_%H%M%S`"
  echo "INFO ($scriptName) - issuing command: '$theCommand'"
  echo 
  eval $theCommand
  echo
 
  # -------------------------------------------------------------
  # Customization complete
  # -------------------------------------------------------------
  echo; echo "$stdSeperatorLine"
  echo 
  echo "SUCCESS: ($scriptName) - 7133 JBOD MES UPGRADE COMPLETE. (`date`)"

  # -------------------------------------------------------------
  # Make user enter a response to continue with revfile
  # -------------------------------------------------------------
  echo
  enterOneToContinue
 
  thisSubroutine="phase2UpdateSystem"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
  
}  # end phase2UpdateSystem

mainProcedure ()
{
  # -------------------------------------------------
  # echo subroutine entry
  # -------------------------------------------------
  echo; echo "$stdSeperatorLine"
  thisSubroutine="mainProcedure"
  echo; echo "INFO ($scriptName) - ENTER subroutine $thisSubroutine at line: $LINENO"

  # ----------------------------------------------------------------
  # Suggest user increase size of screen
  # ----------------------------------------------------------------
  clear 
  echo
  echo '********************************************************************************'
  echo '*'
  echo "* Information will be displayed on the following screens that will"
  echo "*   not fit in a window which was launched with the default size."
  echo '*'
  echo "* You should increase the size of this window in order to effectively"
  echo '* view information presented in this script.'
  echo '*'
  echo "* The window size can be increased by:"
  echo '*'
  echo "*  -- Maximizing to full screen with the maximize button"
  echo "*     (upper right hand corner)"
  echo '*'
  echo "*  -- Stretching the window height. Position the mouse on the"
  echo "*     top or bottom edge of the screen. Notice that the pointer"
  echo "*     changes to indicate the mouse is positioned to stretch the screen."
  echo "*     Hold down the left mouse key and move the mouse to stretch the"
  echo "*     screen."
  echo '*'
  echo "* Hit the Enter Key to continue when the screen size is adjusted"
  echo '*'
  echo '********************************************************************************'
  read

  # --------------------------------------------------
  # prompt user for phase to run
  # --------------------------------------------------
  clear
  echo
  echo '********************************************************************************'
  echo '*'
  echo "* Make selection to run the required phase:"
  echo '*'
  echo '*   1) Run phase 1 to:'
  echo '*'
  echo '*      -- Install the minimum nsm package level required for this MES'
  echo '*'
  echo '*        Note: if this level or a later level is already installed'
  echo '*              the install process will simply NO-OP the install'
  echo '*'
  echo '*      -- Run verification checks against the current disks'
  echo '*'
  echo '*      Phase 1 is run prior to modifying the 3466 system. Problems'
  echo '*      encountered MUST be corrected prior to continuing.'
  echo '*'
  echo '*   2) Run phase 2 to configure the disks added to the system'
  echo '*'
  echo '*   3) Exit script ...'
  echo '*'
  echo '* NOTE: Both phase 1 and phase 2 can be re-run in the event the script'
  echo '*       does not complete in a previous run'
  echo '*'
  echo '********************************************************************************'
  echo
  sleep 1 # seems to be required, or output gets messed up
  PS3='
Enter Choice:'
  select userResponse in \
      'Phase 1' \
      'Phase 2' \
      'exit'
  do
    case $REPLY in
      1) :
         echo
         echo 'User entered option 1, run phase 1'
         phase1GetReady
         ;;
      2) :
         echo
         echo 'User entered option 2, run phase 2'
         phase2UpdateSystem
         ;;
      3) :
         echo
         echo 'User entered option 3, exit'
         echo
         enterOneToExit
         ;;
      *) echo
         echo "Invalid response: '$REPLY'. Try again."
         echo
         ;;
    esac

    if [[ -n $userResponse ]] ; then
      break
    fi
  done

  thisSubroutine="mainProcedure"
  echo; echo "INFO ($scriptName) - EXIT subroutine $thisSubroutine at line: $LINENO"
 
}  # end mainProcedure
 
# *****************************************************************************
# Mainline
# *****************************************************************************

export scriptName='h27366a1.ec'
progressLogFs='/adsmlog'
progressLogDir="$progressLogFs/$scriptName"
if [ -d $progressLogDir ]
then
  : 
else
  mkdir -p $progressLogDir >/dev/null 2>&1
  mkdirRc=$?
  if (( $mkdirRc != 0 ))
  then
    echo 
    echo "NSM-95450-E: ERROR ($scriptName) unable to create the directory, $progressLogDir"
    echo 
    echo "  Check for a full file system with the command 'df $progressLogFs'"
    echo 
    echo "  Check to see if the name already exists as a file that is not a directory"
    echo "    with the command, ls -l '$progressLogDir'"
    echo 
    echo "  Corrective action required. Exiting ...."
    echo 
    enterOneToExit
  fi
fi  
export progressLog="$progressLogDir/progressLog"

# -------------------------------------------------------------------------
# Process special options.
#   1) Special options must start with a '-'
#   2) The special option must be adjacent to the '-' and can be any
#      number of characters
#   3) Any number of values can be specified for a special option.
#      If values are processed they must be captured and then shifted left
#   4) All special options must be specified prior to positional parameters
# -------------------------------------------------------------------------
firstCharParm1=`echo $1 | cut -b1`
while [ "$firstCharParm1" = '-' ]
do
  specialOption=`echo $1 | cut -b2-`
  case $specialOption in
    ('h')
      helpOption='yes'
      : ;;
    ('d') # development option, comment our for production
      vadminQuitOption='-q'
      adminIDin='admin'
      adminPasswordIn='admin'
      : ;;
    (*)
      writeErr
      writeErr "NSM-95447-E: ERROR ($scriptName) special option: '$1' not recognized"
      writeErr
      programRc=1
      enterOneToExit
      exit $programRc
      : ;;
  esac
  shift 1
  firstCharParm1=`echo $1 | cut -b1`
done

# -----------------------------------------------------------------------------
# Write help text as appropriate
# -----------------------------------------------------------------------------
if [ "$helpOption" = 'yes' ]
then
  clear
  cat $0 | grep \#@ | grep -v 'grep #@' | more
  exit
fi


# ----------------------------------------------------------------
# InitializeStuff
# ----------------------------------------------------------------
export stdSeperatorLine="*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*"

# ------------------------------------------------------------
# Make sure the /cd directory structure looks sane
# ------------------------------------------------------------
if [ ! -d /cd/buildscr ]
then

  echo 
  echo "NSM-95437-E: ERROR - ($scriptName): directory /cd/buildscr does not exist."
  echo
  checkIfCdMounted=`df /cd | grep -v Filesystem | awk '{print $7}'`
  if [ "$checkIfCdMounted" = '/cd' ]
  then
    echo "The /cd filesystem is mounted. Check if the correct cd is inserted"
    echo
    unmountCd='unmount /cd'
    echo "Issuing command: '$unmountCd' to unmount the /cd filesystem"
    echo
    eval $unmountCd
    echo
    echo "Issuing command: 'df /cd' to verify /cd filesystem is unmounted"
    echo
    df /cd
    echo
    echo "Note: the filesystem may not unmount if someone is using this directory"
    echo "      typically, this happens when some system had "cdED" into the directory"
    echo
  else
    echo "The /cd filesystem is NOT mounted."
  fi
  echo
  echo "Corrective action required. Exiting ...."
  echo
  echo "Exiting ..."
  echo
  enterOneToExit

else

  # -----------------------------------------------------
  # Add the main script directory to the end of the path
  # -----------------------------------------------------
  mesBuildscrDir='/cd/buildscr'
  export PATH="$PATH:/cd/buildscr"

  # -----------------------------------------------------
  # Ensure current path starts with required directories
  # -----------------------------------------------------
  currentPath=`env PATH`
  firstCurrentPathEntry=`echo $currentPath | cut -f1 -d':'`
  secondCurrentPathEntry=`echo $currentPath | cut -f2 -d':'`

  # -----------------------------------------------------
  # Add directories to the beginning of the path as required
  # -----------------------------------------------------
  temp3466bin='/tmp/3466bin'
  if [ "$firstCurrentPathEntry" = "$temp3466bin" ]
  then # OK, this is what we need
    if [ "$secondCurrentPathEntry" = '.' ] || [ "$secondCurrentPathEntry" = './' ]
    then
      echo "INFO: PATH already contains $temp3466bin."
    else
       restOfPath=`echo $currentPath | cut -f2- -d':'`
       echo "INFO: adding $temp3466bin to PATH"
       export PATH="$temp3466bin:./:$restOfPath"
    fi
  else
    if [ "$firstCurrentPathEntry" = '.' ] || [ "$firstCurrentPathEntry" = './' ]
    then # OK, this is what we need
      newPath="$temp3466bin:$currentPath"
    else
      newPath="$temp3466bin:./:$currentPath"
    fi
    echo "INFO: adding $temp3466bin to PATH"
    export PATH="$newPath"
  fi

  buildid=$(date "+%m%d_%H%M")  # mmdd_hhmm


  # --------------------------------------------------------------------
  # setup logging
  # --------------------------------------------------------------------
  timeStamp=`date +%m%d%Y_%H%M%S`
  export logDir="/tmp/$scriptName"
  mkdir -p $logDir >/dev/null 2>&1
  theConsoleLog="$logDir/$scriptName.consoleLog$timeStamp"

  # --------------------------------------------------------------------
  # setup trap for CNTL-C
  # --------------------------------------------------------------------
  scriptStatus='UNSUCCESSFULL (`date`) - SCRIPT DID NOT COMPLETE' # changed to SUCCESSFULL when appropriatte
  userInteruptString="
  INFO: ($scriptName) - user entered CNTL-C to terminate program

  INFO: ($scriptName) script status at exit: $scriptStatus"
  trap 'echo "$userInteruptString" | tee -a "$theConsoleLog"' INT

  # --------------------------------------------------------------------
  # Invoke the mainProcedure to do the bulk of the work
  # --------------------------------------------------------------------
  cd $mainScriptDir
  mainProcedure $scriptName 2>&1 | tee $theConsoleLog

  # --------------------------------------------------------------------
  # let user review the console log
  # --------------------------------------------------------------------
  archiveDirectory="/adsmlog/$scriptName"
  mkdir -p $archiveDirectory >/dev/null 2>&1

  revfile -c -a $archiveDirectory $theConsoleLog

  # --------------------------------------------------------------------
  # say goodbye
  # --------------------------------------------------------------------
  exitTimeStamp=`date +"%D %T"`
  echo
  echo "INFO: ($scriptName) - EXITING at $exitTimeStamp"

  return $programRC

fi

